/*This program is for doing crystalline umbrella sampling of the cannonical ensemble on the sphere*/

/*includes*/
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"monte_carlo_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"read_h.h"
#include"running_average_h.h"
#include"energy_h.h"
#include"histograms_h.h"
#include"histograms2D_h.h"
#include"bond_order_6h.h"
#include"neighbours_h.h"
#include"cluster_moves_h.h"
#include"umbrella_h.h"
#include"bond_order_6h.h"
#include"bond_order6_plane_h.h"

int main (int argc, char *argv[]) {

/*validation*/
if (argc != 2) die ("This program requires one argument, the run ID");

/*system structs*/
Mc_state state;
Mc_state old_state;
Mc_cell cell;
Mc_cell old_cell;
Mc_clusters cluster;
Data input;
Mc_umbrella umbrella;
Mc_average energy_average;
Mc_histogram energy_histogram;
Mc_average bond_average;
Mc_histogram bond_histogram;

/*read options file*/
char command[50];
FILE *fp;
sprintf(command, "options%s.txt",argv[1]);
fp = fopen(command,"r");
READ_options(&input,fp);
fclose(fp);

/*seed the random number generator*/
seedMT(input.seed);

/*fix file*/
sprintf(command,"mkdir run%s",argv[1]);
system(command);
sprintf(command,"cp options%s.txt run%s", argv[1],argv[1]);
system(command);
sprintf(command, "run%s",argv[1]);
chdir(command);

int i,j;
/*set up variables for stuff that'll be used a lot (for speed)*/
long max_steps = input.max_steps;
long record_rate = input.record_rate;
long average_rate = input.average_rate;
long frame_rate = input.frame_rate;
long equilibriate = input.equilibriate;
int periodic = input.periodic;
double delta_r = input.delta_r;
double deltaC_move = input.deltaC_move;
double deltaC_spin = input.deltaC_spin;
double psTemp_max = 4*input.paramsP[0];
double psTemp_min = input.paramsP[0];
double *paramsP = input.paramsP;
double *paramsG = input.paramsG;
double neighbour_cutoff = input.neighbour_cutoff;
double lambda = input.lambda;
int n_target = input.n_target;
long umbrella_rate = input.umbrella_rate;
double order_cutoff = input.order_cutoff;
void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
void (*random) (double *x, double *y, double *z, double *params);
double (*get_area) (double params[]);
double (*potential) (double distance, double params[]);
void (*move_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
				int head,double delta,double *paramsG);
void (*spin_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
				int head,double delta,double *paramsG);
double * (*BO6_calculate) (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff);
move = input.move;
random = input.random;
potential = input.potential;
get_area = input.get_area;
move_cluster = input.move_cluster;
spin_cluster = input.spin_cluster;
BO6_calculate = input.BO6_calculate;

/*initialise*/
mc_initialise(&state,input.read_file,input.restart,input.n_particles_initial,
		paramsG,argv[1],random,&delta_r,&deltaC_move,&deltaC_spin);
CLL_init(&cell, input.cell_box_edge, input.cell_min_size);
CLL_init_particles(&cell,&state);
CMV_init(&cluster,state.n_particles);

double psudo_temp = CMV_get_psudo_temp(psTemp_max,psTemp_min);
CMV_get_clusters(&state,&cell,&cluster,psudo_temp,periodic,paramsG,potential,paramsP);
MCS_state_clone(&old_state,&state);
CLL_cell_clone(&old_cell,&cell,state.n_particles);
int nucleus = 0;
int old_nucleus = 10*state.n_particles;

UMB_new_histogram(&umbrella,get_area(paramsG),"Nucleus",input.restart);
RA_new_average(&energy_average, "energy",max_steps/record_rate,input.restart);
RA_new_average(&bond_average,"bond", max_steps/record_rate,input.restart);
HG_new_histogram(&energy_histogram,0.0,-8.0,0.1,"energy",input.restart);
HG_new_histogram(&bond_histogram,1.4,0.0,0.01,"bond",input.restart);
printf("hi\n");
float rm;
/*------------------------------equilibriate-------------------------------------*/
int total_moves;
int moves_accepted;
int totalC_moves;
int Cmoves_accepted;
int total_spins;
int spins_accepted;
long steps = 0;
double *order;
while (steps < equilibriate) {
	total_moves = 0;
	moves_accepted = 0;
	totalC_moves = 0;
	Cmoves_accepted = 0;
	total_spins = 0;
	spins_accepted = 0;

	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		if (rm < 0.985) {
			total_moves++;
			if(mc_particle_move(&state,&cell,periodic,
					    delta_r,potential, move, paramsP, paramsG)) {
				moves_accepted++;
			}
		} else if (rm < 0.99) {
			psudo_temp = CMV_get_psudo_temp(psTemp_max,psTemp_min);
			CMV_get_clusters(&state,&cell,&cluster,psudo_temp,
					 periodic,paramsG,potential,paramsP);
		} else if (rm < 0.995) {
			totalC_moves ++;
			if( CMV_move_mc (&state,&cell,&cluster,deltaC_move,paramsG,psudo_temp,
					 periodic,paramsG,potential,move_cluster,paramsP)) {
				Cmoves_accepted ++;
			}
		} else {
			total_spins++;
			if( CMV_spin_mc(&state,&cell,&cluster,deltaC_spin,paramsG,psudo_temp,
					periodic,paramsG,potential,spin_cluster,paramsP)) {
				spins_accepted++;
			}
		}
	}

	if (steps > equilibriate/2) {
		order = NEI_double(&state,paramsG,neighbour_cutoff,periodic);
		UMB_move_largest(&state, &old_state, &cell, &old_cell, &nucleus, &old_nucleus,
		                  lambda, n_target, order_cutoff, neighbour_cutoff, order, 
			          periodic, paramsG);
		free(order);
	}

	steps += umbrella_rate;
	if ((float) moves_accepted/total_moves > 0.25) {
		if (delta_r < paramsG[0]) delta_r *= 1.05;
	} else if ((float) moves_accepted/total_moves < 0.25) {
		delta_r /= 1.05;
	}

	if ((float) Cmoves_accepted/totalC_moves > 0.25) {
		if (deltaC_move < 2*M_PI) deltaC_move *= 1.05;
	} else if ((float) Cmoves_accepted/totalC_moves < 0.25) {
		deltaC_move /= 1.05;
	}

	if ((float) spins_accepted/total_spins > 0.25) {
		if (deltaC_spin < 2*M_PI) deltaC_spin *= 1.05;
	} else if ((float) spins_accepted/total_spins < 0.25) {
		deltaC_spin /= 1.05;
	}

	if (steps % frame_rate == 0) {
		mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
	}
}
steps = 0;
/*---------------------------simulation loop-------------------------------------*/
int count_records = 0;
double *en_temp;
double *bond_temp;
double *nei_temp;
double en_tot;
double bond_tot;
double *cluster_temp;

while (steps < max_steps) {
	
	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		if (rm < 0.945) {
			mc_particle_move(&state,&cell,periodic,delta_r,potential, move, paramsP,
					 paramsG);
		} else if (rm < 0.95) {
			psudo_temp = CMV_get_psudo_temp(psTemp_max,psTemp_min);
			CMV_get_clusters(&state,&cell,&cluster,psudo_temp,
					 periodic,paramsG,potential,paramsP);
		} else if (rm < 0.975) {
			CMV_move_mc (&state,&cell,&cluster,deltaC_move,paramsG,psudo_temp,
					 periodic,paramsG,potential,move_cluster,paramsP);
		} else {
			CMV_spin_mc (&state,&cell,&cluster,deltaC_spin,paramsG,psudo_temp,
					 periodic,paramsG,potential,move_cluster,paramsP);
		}
	}

	order = NEI_double (&state, paramsG, neighbour_cutoff,periodic);
	UMB_move_largest(&state, &old_state, &cell, &old_cell, &nucleus, &old_nucleus,
		                  lambda, n_target, order_cutoff, neighbour_cutoff, order, 
			          periodic, paramsG);;
	free(order);
	steps += umbrella_rate;
	if (steps %average_rate == 0) {
		count_records++;

		/*-----------------do experiments----------------*/
		en_temp = EGY_energy_list (&state, &cell,periodic,paramsG, potential, paramsP);
		en_tot = array_sum_d(en_temp,state.n_particles);
		bond_temp = BO6_calculate(&state,&cell, paramsG, neighbour_cutoff);
		bond_tot = array_sum_d(bond_temp,state.n_particles)/state.n_particles;
		HG_add_array(&energy_histogram,en_temp,state.n_particles);
		HG_add_array(&bond_histogram,bond_temp,state.n_particles);
		UMB_add_value(&umbrella,nucleus);
		RA_add_average(&energy_average,en_tot);
		RA_add_average(&bond_average,bond_tot);

		if (steps % record_rate == 0) {
			RA_push_average(&energy_average);
			RA_push_average(&bond_average);
		}
	
		if (steps % frame_rate == 0) {
			mc_record(&state);
			mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
			RA_print_average(&energy_average);
			RA_print_average(&bond_average);
			HG_print_histogram(&energy_histogram);
			HG_print_histogram(&bond_histogram);
			UMB_print(&umbrella);
			nei_temp = NEI_double(&state,paramsG,neighbour_cutoff,periodic);
			cluster_temp = UMB_record_largest(&state,order_cutoff,neighbour_cutoff,
								bond_temp, periodic,paramsG);
			mc_record_order( nei_temp, state.n_particles,"neighbours");
			mc_record_order( bond_temp, state.n_particles, "bond_order");
			mc_record_order( cluster_temp, state.n_particles, "cluster");
			free(nei_temp);
			free(cluster_temp);
		}

		free(en_temp);
		free(bond_temp);

		//just in case rounding errors//
		if (count_records > (max_steps/average_rate) - 1) break;
	}
}

/*-------------------------save the data----------------------------*/

fp = fopen("data.txt","w");
fprintf(fp,"Records %d",count_records);
fclose(fp);
mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
umbrella.isScaled = 1;
UMB_scale(&umbrella,lambda,n_target);

RA_print_average(&energy_average);
RA_print_average(&bond_average);
HG_print_histogram(&energy_histogram);
HG_print_histogram(&bond_histogram);
UMB_print(&umbrella);

CLL_end(&cell);
CMV_destroy(&cluster);
MCS_end(&state);
RA_destroy(&energy_average);
RA_destroy(&bond_average);
HG_destroy(&energy_histogram);
HG_destroy(&bond_histogram);
READ_destroy(&input);
UMB_destroy(&umbrella);

return 0;
}
