#include"mc_state_h.h"
#include"cell_list_h.h"
#include"monte_carlo_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"read_h.h"
#include"running_averageRE_h.h"
#include"energy_h.h"
#include"histogramsRE_h.h"
#include"histograms2D_h.h"
#include"bond_order_6h.h"
#include"neighbours_h.h"
#include"cluster_moves_h.h"
#include"umbrellaRE_h.h"
#include"bond_order_6h.h"
#include"bond_order6_plane_h.h"
#include"replica_exchange_h.h"

int main (void) {

/*system structs*/
Mc_state state;
Mc_state old_state;
Mc_cell cell;
Mc_cell old_cell;
Mc_clusters cluster;
Data input;
int rank;
int size;

/*initialise the mpi library*/
RE_initialise(&rank,&size);
int i_sim = rank;

Mc_umbrella *umbrella = malloc(sizeof(Mc_umbrella)*size);
Mc_umbrella umbrella_total;
/*read options file*/
char command[50];
FILE *fp;
sprintf(command, "options%d.txt",rank);
fp = fopen(command,"r");
READ_options(&input,fp);
fclose(fp);
/*seed the random number generator*/
seedMT(input.seed);

/*fix file*/
sprintf(command,"mkdir run%d",rank);
system(command);
sprintf(command,"cp options%d.txt run%d", rank, rank);
system(command);
sprintf(command, "run%d",rank);
chdir(command);

int i,j;
/*set up variables for stuff that'll be used a lot (for speed)*/
long max_steps = input.max_steps;
long record_rate = input.record_rate;
long average_rate = input.average_rate;
long frame_rate = input.frame_rate;
long equilibriate = input.equilibriate;
int periodic = input.periodic;
double delta_r = input.delta_r;
double deltaC_move = input.deltaC_move;
double deltaC_spin = input.deltaC_spin;
double psTemp_max = 4*input.paramsP[0];
double psTemp_min = input.paramsP[0];
double *paramsP = input.paramsP;
double *paramsG = input.paramsG;
double activity = input.activity;
double neighbour_cutoff = input.neighbour_cutoff;
double lambda = input.lambda;
double n_target = (double) input.n_target;
long umbrella_rate = input.umbrella_rate;
double order_cutoff = input.order_cutoff;
void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
void (*random) (double *x, double *y, double *z, double *params);
double (*get_area) (double params[]);
double (*potential) (double distance, double params[]);
void (*move_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
				int head,double delta,double *paramsG);
void (*spin_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
				int head,double delta,double *paramsG);
double * (*BO6_calculate) (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff);
move = input.move;
random = input.random;
potential = input.potential;
get_area = input.get_area;
move_cluster = input.move_cluster;
spin_cluster = input.spin_cluster;
BO6_calculate = input.BO6_calculate;
double *delta_list = malloc(sizeof(double));
sprintf(command, "%d", i_sim);
mc_initialise(&state,input.read_file,input.restart,input.n_particles_initial,
		paramsG,command,random,&delta_r,&deltaC_move,&deltaC_spin);
CLL_init(&cell, input.cell_box_edge, input.cell_min_size);
CLL_init_particles(&cell,&state);
CMV_init(&cluster,state.n_particles);
double psudo_temp = CMV_get_psudo_temp(psTemp_max,psTemp_min);

MCS_state_clone(&old_state,&state);
CLL_cell_clone(&old_cell,&cell,state.n_particles);
int nucleus = 0;
int old_nucleus = state.n_particles;

for (i = 0; i < size; i++) {
	sprintf(command, "nucleus_re%d",i);
	UMB_new_histogram(umbrella+i,get_area(paramsG),command,input.restart);
}
UMB_new_histogram(&umbrella_total,get_area(paramsG),"Nucleus",input.restart);
float rm;
/*------------------------------equilibriate-------------------------------------*/
int total_moves;
int moves_accepted;
int total_exchanges;
int exchanges_accepted_up;
int exchanges_accepted_down;

long steps = 0;
double *order;
while (steps < equilibriate) {
	total_moves = 0;
	moves_accepted = 0;
	total_exchanges = 0;
	exchanges_accepted_up = 0;
	exchanges_accepted_down = 0;

	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		if (rm < 0.5) {
			total_moves ++;
			if (mc_particle_move(&state,&cell,periodic,
					    delta_r,potential, move, paramsP, paramsG)) {
				moves_accepted++;
			}
		} else if (rm < 0.75) {
			mc_particle_add (&state,&cell,periodic,potential,random,paramsP,
						paramsG,activity);
		} else {
			mc_particle_remove(&state,&cell,periodic,potential,
						 paramsP, paramsG, activity);
		}
	}	

	if (steps > umbrella_rate * 100) {
		order = NEI_double (&state, paramsG, neighbour_cutoff,periodic);
		UMB_move_largest(&state, &old_state, &cell, &old_cell, &nucleus, &old_nucleus,
				 lambda, (int) n_target, order_cutoff, neighbour_cutoff, order, 
			          periodic, paramsG);
		free(order);
	}	
	steps += umbrella_rate;
	
	/*exchange*/
	if (steps % (2*umbrella_rate) == 0) {
		total_exchanges++;
		delta_list[0] = delta_r;
		if (steps % (4*umbrella_rate) == 0) {
			if (rank % 2 == 0 && rank != 0) {
				if (RE_exchange_down(nucleus*lambda,
					&n_target,delta_list,1,rank,&i_sim)) {
					exchanges_accepted_down ++;
				}
			} else if (rank % 2 == 1 && rank != size-1) {
			        if(RE_exchange_up(nucleus*lambda,&n_target,
					delta_list,1,rank, &i_sim,RE_accept_n_target)) {
					exchanges_accepted_up ++;	
				}
			} 
		} else {
			if (rank % 2 == 0 && rank != size-1) {
			        if (RE_exchange_up(nucleus*lambda,&n_target,
					delta_list,1,rank, &i_sim,RE_accept_n_target)) {
					exchanges_accepted_up ++;
				}
			} else if (rank % 2 == 1 && rank != 0) {
				if(RE_exchange_down(nucleus*lambda,&n_target,
							delta_list,1,rank,&i_sim)) {
					exchanges_accepted_down ++;	
				}
			} 
		}
		delta_r = delta_list[0];
	}

	if ((float) moves_accepted/total_moves > 0.25) {
		if (delta_r < paramsG[0]) delta_r *= 1.05;
	} else if ((float) moves_accepted/total_moves < 0.25) {
		delta_r /= 1.05;
	}	
	
	if (steps % frame_rate == 0) {
	        sprintf(command,"%d",i_sim);
		RE_mc_save(&state,command,delta_r,0.0,0.0);
	}
	
}
/*----------------------------simulation loop----------------------------------*/
steps = 0;
int count_records = 0;
int *nuclei_int;
double *nei_temp;
double *cluster_temp;
while (steps < max_steps) {
	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		if (rm < 0.5) {
			mc_particle_move(&state,&cell,periodic,
					    delta_r,potential, move, paramsP, paramsG);
		} else if (rm < 0.75) {
			mc_particle_add (&state,&cell,periodic,potential,random,paramsP,
						paramsG,activity);
		} else {
			mc_particle_remove(&state,&cell,periodic,potential,
						 paramsP, paramsG, activity);
		}
	}

	order = NEI_double (&state, paramsG, neighbour_cutoff,periodic);
	UMB_move_largest(&state, &old_state, &cell, &old_cell, &nucleus, &old_nucleus,
			  lambda, (int) n_target, order_cutoff, neighbour_cutoff, order, 
			   periodic, paramsG);
	free(order);

	steps += umbrella_rate;
	/*exchange*/
	if (steps % (2*umbrella_rate) == 0) {
		total_exchanges++;
		delta_list[0] = delta_r;
		if (steps % (4*umbrella_rate) == 0) {
			if (rank % 2 == 0 && rank != 0) {
				if (RE_exchange_down(nucleus*lambda,
					&n_target,delta_list,1,rank,&i_sim)) {
					exchanges_accepted_down ++;
				}
			} else if (rank % 2 == 1 && rank != size-1) {
			        if(RE_exchange_up(nucleus*lambda,&n_target,
					delta_list,1,rank, &i_sim,RE_accept_n_target)) {
					exchanges_accepted_up ++;	
				}
			} 
		} else {
			if (rank % 2 == 0 && rank != size-1) {
			        if (RE_exchange_up(nucleus*lambda,&n_target,
					delta_list,1,rank, &i_sim,RE_accept_n_target)) {
					exchanges_accepted_up ++;
				}
			} else if (rank % 2 == 1 && rank != 0) {
				if(RE_exchange_down(nucleus*lambda,&n_target,
							delta_list,1,rank,&i_sim)) {
					exchanges_accepted_down ++;	
				}
			} 
		}

		delta_r = delta_list[0];
	}

	if (steps % average_rate == 0) {
		order = NEI_double (&state, paramsG, neighbour_cutoff,periodic);
		count_records++;
		nuclei_int = UMB_get_nucleus(&state,order_cutoff,neighbour_cutoff,
				order,periodic,paramsG);

		UMB_add_array(umbrella+i_sim,nuclei_int,state.n_particles);


		UMB_add_array_scale(umbrella+i_sim,nuclei_int,
				state.n_particles,lambda,n_target,nucleus);


		if (steps % frame_rate == 0) {
			mc_record(&state);
			sprintf(command,"%d",i_sim);
			RE_mc_save(&state,command,delta_r,0.0,0.0);
			for (i = 0; i < size; i++) {
				UMB_print(umbrella+i);
			}
			nei_temp = NEI_double(&state,paramsG,neighbour_cutoff,periodic);
			cluster_temp = UMB_record_largest(&state,order_cutoff,neighbour_cutoff,
								order, periodic,paramsG);
			mc_record_order( nei_temp, state.n_particles,"neighbours");
			mc_record_order( cluster_temp, state.n_particles, "cluster"); 
			free(nei_temp);
			free(cluster_temp);
		}
		//just in case rounding errors//

		if (count_records > (max_steps/average_rate) - 1) break;
		free(nuclei_int);
		free(order);
	}
}

sprintf(command,"%d",i_sim);
RE_mc_save(&state,command,delta_r,deltaC_move,deltaC_spin);
for (i = 0; i < size; i++) {
	UMB_print(umbrella+i);
}

fp = fopen("data.txt","w");
 fprintf(fp,"accept %f %f\n",(double) exchanges_accepted_up/(0.5*total_exchanges)
			    ,(double) exchanges_accepted_down/(0.5*total_exchanges));
fclose(fp);

UMB_sort_histogram_scale(&umbrella_total,umbrella,size);
umbrella_total.isScaled = 1;
UMB_print(&umbrella_total);

CLL_end(&cell);
CMV_destroy(&cluster);
MCS_end(&state);
READ_destroy(&input);
UMB_destroy(&umbrella_total);

 for (i = 0; i < size; i++) {
   UMB_destroy(umbrella+i);
 }

 free(umbrella);

RE_end();
return 0;
}
