/* This is a mpdule for calculating the bond order parameter for partices on a sphere.
 *
 * It assumes that each particle has six nearest neighbours (so it is a measure of defects as well
 * as order.
 *
 * The prefix is BO6_
 * */

#include"mc_state_h.h"
#include"std_h.h"
#include"mc_math_h.h"
#include"bond_order_6h.h"


/*FUNCTION: bons order calculate: calculates the bond order parameter of each particle
 *
 *Return: pointer to a list of bond_orders
 *input: state: the current state of the system
 *input radius: the radius of the input sphere*/
double * BO6_calculate (Mc_state *state, Mc_cell *cell, double *radius, double cutoff) {

	/*dummies*/
	int i,j,count,first,nn,n;
	double xo, yo, zo, xi, yi, zi, dist;
	double xd, yd, zd, xref, yref, zref, sref, scale, angle ,dot;
	double RE, IM, order_re =0, order_im = 0;
	int current_cell;

	/*return list*/
	double *orders = malloc (sizeof(double)*state->n_particles);

	/*loop over all particles and get the bond order*/
	for (i = 0; i < state->n_particles; i++) {
		/*set up*/
		RE = 0;
		IM = 0;
		nn = 0;
		first = 1;
		
		xo = state->x[i];
		yo = state->y[i];
		zo = state->z[i];

		/*get neighbours*/
		current_cell = cell->of_particle[i];
		n = cell->head_of[current_cell];
		while (n != index && n!= -1) {
			/*do work*/
			xi = state->x[n];
			yi = state->y[n];
			zi = state->z[n];

			dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
			if (dist < cutoff) {
				xd = xo-xi;
				yd = yo-yi;
				zd = zo-zi;
				nn++;

				/*do projection*/
				scale = (xd*xo+yd*yo+zd*zo)/((*radius)*(*radius));
				xd = xd-scale*xo;
				yd = yd-scale*yo;
				zd = zd-scale*zo;
				dist = sqrt((xd*xd+yd*yd+zd*zd));

				if (first) {
					xref = xd;
					yref = yd;
					zref = zd;
					sref = sqrt(xref*xref+yref*yref+zref*zref);
					first = 0;
				}
				dot = xd*xref+yd*yref+zd*zref;
				if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
				if (dot/(dist*sref) < -1.0) {
					angle = M_PI;
				} else if (dot/(dist*sref) > 1.0) {
					angle = 0;
				} else {
					angle = acos(dot/(dist*sref));
				}
				RE += cos(6*angle);
				IM += sin(6*angle); 
			}

			n = cell->ll[n];
		}		

		/*skip one*/
		if (n != -1) {
			n = cell->ll[n];
		}

		while (i != -1) {
			/*do work*/
			xi = state->x[n];
			yi = state->y[n];
			zi = state->z[n];

			dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
			if (dist < cutoff) {
				xd = xo-xi;
				yd = yo-yi;
				zd = zo-zi;
				nn++;

				/*do projection*/
				scale = (xd*xo+yd*yo+zd*zo)/((*radius)*(*radius));
				xd = xd-scale*xo;
				yd = yd-scale*yo;
				zd = zd-scale*zo;
				dist = sqrt((xd*xd+yd*yd+zd*zd));

				if (first) {
					xref = xd;
					yref = yd;
					zref = zd;
					sref = sqrt(xref*xref+yref*yref+zref*zref);
					first = 0;
				}
				dot = xd*xref+yd*yref+zd*zref;
				if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
				if (dot/(dist*sref) < -1.0) {
					angle = M_PI;
				} else if (dot/(dist*sref) > 1.0) {
					angle = 0;
				} else {
					angle = acos(dot/(dist*sref));
				}
				RE += cos(6*angle);
				IM += sin(6*angle); 
			}
			n = cell->ll[n];
		}
		
		/*do other cells*/
		for (j = 0; j < 26; j++) {
			if (cell->neighbours[cell->of_particle[i]][j] == -1) break;
			current_cell = cell->neighbours[cell_of][j];
			n = cell->head_of[current_cell];
			while (n != -1) {
				/*do work*/
				xi = state->x[n];
				yi = state->y[n];
				zi = state->z[n];

				dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
				if (dist < cutoff) {
					xd = xo-xi;
					yd = yo-yi;
					zd = zo-zi;
					nn++;

					/*do projection*/
					scale = (xd*xo+yd*yo+zd*zo)/((*radius)*(*radius));
					xd = xd-scale*xo;
					yd = yd-scale*yo;
					zd = zd-scale*zo;
					dist = sqrt((xd*xd+yd*yd+zd*zd));

					if (first) {
						xref = xd;
						yref = yd;
						zref = zd;
						sref = sqrt(xref*xref+yref*yref+zref*zref);
						first = 0;
					}
					dot = xd*xref+yd*yref+zd*zref;
					if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
					if (dot/(dist*sref) < -1.0) {
						angle = M_PI;
					} else if (dot/(dist*sref) > 1.0) {
						angle = 0;
					} else {
						angle = acos(dot/(dist*sref));
					}
					RE += cos(6*angle);
					IM += sin(6*angle); 
			}
				n = cell->ll[n];
			}
		}

		/*collect*/
		if (nn >= 2) {
			RE = RE/6;
			IM = IM/6;
			orders[i] = sqrt(RE*RE+IM*IM);
		} else {
			orders[i] = 0;
		}

	}
	return orders;	
}


/* This is a mpdule for calculating the bond order parameter for partices on a sphere.
 *
 * It assumes that each particle has six nearest neighbours (so it is a measure of defects as well
 * as order.
 *
 * The prefix is BO6_
 * */

#include"mc_state_h.h"
#include"std_h.h"
#include"mc_math_h.h"
#include"bond_order_6h.h"


/*FUNCTION: bons order calculate: calculates the bond order parameter of each particle
 *
 *Return: pointer to a list of bond_orders
 *input: state: the current state of the system
 *input radius: the radius of the input sphere*/
double * BO6_calculate (Mc_state *state, Mc_cell *cell, double *radius, double cutoff) {

	/*dummies*/
	int i,j,count,first,nn;
	double xo, yo, zo, xi, yi, zi, dist;
	double xd, yd, zd, xref, yref, zref, sref, scale, angle ,dot;
	double RE, IM, order_re =0, order_im = 0;

	/*return list*/
	double *orders = malloc (sizeof(double)*state->n_particles);

	/*loop over all particles and get the bond order*/
	for (i = 0; i < state->n_particles; i++) {
		/*set up*/
		RE = 0;
		IM = 0;
		nn = 0;
		first = 1;
		
		xo = state->x[i];
		yo = state->y[i];
		zo = state->z[i];

		/*get neighbours*/
		for (j = 0; j < state->n_particles; j++) {
			if (i == j) continue;
			xi = state->x[j];
			yi = state->y[j];
			zi = state->z[j];

			dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
			if (dist < cutoff) {
				xd = xo-xi;
				yd = yo-yi;
				zd = zo-zi;
				nn++;

				/*do projection*/
				scale = (xd*xo+yd*yo+zd*zo)/((*radius)*(*radius));
				xd = xd-scale*xo;
				yd = yd-scale*yo;
				zd = zd-scale*zo;
				dist = sqrt((xd*xd+yd*yd+zd*zd));

				if (first) {
					xref = xd;
					yref = yd;
					zref = zd;
					sref = sqrt(xref*xref+yref*yref+zref*zref);
					first = 0;
				}
				dot = xd*xref+yd*yref+zd*zref;
				if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
				if (dot/(dist*sref) < -1.0) {
					angle = M_PI;
				} else if (dot/(dist*sref) > 1.0) {
					angle = 0;
				} else {
					angle = acos(dot/(dist*sref));
				}
				RE += cos(6*angle);
				IM += sin(6*angle); 
			}
			
		}
		/*collect*/
		if (nn >= 2) {
			RE = RE/6;
			IM = IM/6;
			orders[i] = sqrt(RE*RE+IM*IM);
		} else {
			orders[i] = 0;
		}

	}
	return orders;	
}
