#ifndef DIPOLE_TS_H
#define DIPOLE_TS_H

/* a module for implenting the dipole potential
 * CANONICAL ENSEMBLE ONLY (owing to density dependence of gamma)
 * the module prefix if dp_
 **/

/*FUNCTION: get params: A helper function to set up the params array for this function to be
 * passed. Not required but useful, as it does all the differentiating for you etc.
 *
 * Output: double params[] (array of size 5)
 *		params[0] = A, the inverse temperature in reduced units
 *
 * Input: double Gamma: The value of the single parameter
 * Input: n_particles the number of particles in the simulation 
 * input : area, the total surface area of the shape.	
 */

void  dp_get_params(double params[], double gamma, int n_particles, double area);

/*
 FUNCTION: double lennard_jones: calculates the pairwise ld potential
 * OUTPUT: ld potential in units of kT
 * INPUT: double distance the separation of the two particles
 */

double dp_potential(double distance, double params[]);


#endif
