/*FUNCTION CONSTRUCT : make a new struct to store all this
 *params function: the struct to be initialised
 * for replica exchange
 */

#include"std_h.h"
#include"mc_math_h.h"
#include"energy_angleRE_h.h"
#include"mpi_h.h"

void EA_new_function(EA_function *function) {

 int n_bins = (int) (M_PI/0.03);
 double bin_size = M_PI/n_bins;
 function->freq = malloc(sizeof(long)*n_bins);
 function->energies = malloc(sizeof(double)*n_bins);
 function->n_bins = n_bins;
 function->bin_size = bin_size;
 int i;
 for(i = 0; i < n_bins;i++) {
  function->freq[i] = 0;
  function->energies[i] = 0.0;
 }
}

/*FUNCTION: destroy deallocate the function*/
void EA_destroy(EA_function *function) {
 free(function->freq);
 free(function->energies);
}

/*FUNCTION: add_values. addes the current snapshot to the function
 *
 *param function -> the function to be added to
 *input energy_list
 *input angle list
 **/
void EA_add_value(EA_function *function, double *energy_list, double *angle_list, int length) {
 double place;
 int i,j;
 for(i = 0; i < length; i++) {
  place = function->bin_size;
  for(j = 0; j < function->n_bins;j++) {
   if(angle_list[i] < place) {
    function->freq[j]++;
    function->energies[j] += energy_list[i];
    break;
   } else {
    place += function->bin_size;
   }
  }
 }	
}

/*function print: prints the function to a file*/

void EA_print(EA_function *function) {
 int i;
 FILE *fp;
 fp = fopen("angle_energy.csv", "w");
 for(i = 0; i < function->n_bins; i++) {
  fprintf(fp,"%lf,%lf\n",((i+0.5)*function->bin_size),function->energies[i]/function->freq[i]);
 }
 fclose(fp);

}


/*FUNCTION :sort: sorts the funcitons from each mpi repetition into the correct ranks
 *
 * final_function: an initialised histogtam to have the data sorted into
 * functions: a list of the histgrams for each index
 * size: the total number of MPI ranks
 *
 **/
void EA_sort_functions(EA_function *final_function, EA_function *functions, int size) {
 int i;
 int n_bins = functions[0].n_bins;
 long *temp_freq;
 long *new_freq;
 new_freq = malloc(sizeof(long)*n_bins);
 double *temp_en;
 double *new_en;
 new_en = malloc(sizeof(double)*n_bins);

 for(i=0;i<size;i++) {
  temp_freq = functions[i].freq;
  MPI_Reduce(temp_freq, new_freq, n_bins, MPI_LONG, MPI_SUM, i, MPI_COMM_WORLD);
  temp_en = functions[i].energies;
  MPI_Reduce(temp_en, new_en, n_bins, MPI_DOUBLE, MPI_SUM, i, MPI_COMM_WORLD);
 }
 free(final_function->freq);
 final_function->freq = new_freq;

 free(final_function->energies);
 final_function->energies = new_en;

}





