#ifndef ENERGY_H
#define ENERGY_H

/*This module contains functions for calculating the potential energy felt
 * by various sets of particles. Only pairwise interaction may be handled
 * The form of the potential is defined via input
 *
 *The module prefix is EGY_
 */


/*Function: energy_calc gets the potential energy of a single particle, at coord
 * u,v.
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 *	double x,y,z: The coordinates of the particle
 *	int index: the index of the particle to be measured. If the particle is imaginary
 *	(ie. not in "state"), set index to n_particles+1
 *	int cell_of: the cell occupied by the target particle
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 *	double(*distance_NI) a function which retuens the disteance between two particles
 *		at u1,v1,u2,v2,params
 *	paramsP is the parameter list for the user-defined potential function
 *	paramsD is the parameter list for the user-defined distence function
 */

double EGY_energy_calc(Mc_state *state, Mc_cell *cell, double x, double y, double z, 
			int index, int cell_of, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]);


/*FUNCTION: energy_total is used to calculate the  energy of
 * the entire system in units of epsilon/kt
 *
 * Output: type double: The energy
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 */

double EGY_energy_total(Mc_state *state, Mc_cell *cell, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]) ;

/*Fruction: energy_list returns a list containing the energy of each particle in the system
 *
 * Output: the list of energies
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 * REMEBER TO FREE THE LIST
 **/
double * EGY_energy_list (Mc_state *state, Mc_cell *cell, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]);
#endif
