#ifndef HISTOGRAM2D_H
#define HISTOGRAM2D_H

/*This is a module for building two dimensional histograms where the first dimension 
 * is just a number (like number of particles) and the second is a scalar variable (such as energy)
 *
 * The module prefix if H2D_
 */

typedef struct hg2Dmcstrc {

	double bin_size;
	double var_max;
	double var_min;
	int n_bins;
	int n_max;
	long *freq;
	char name[50];
	

} Mc_hist2D;

/*
 * FUNCTION: Constructor: initialise a new histogram
 *
 * double var_max: the maximum value of the variable
 * double var_min: the minimum value of the variable
 * double bin_size: the (maximum) size of the bins for the variable
 * int n_max: the maximum value of the number
 * name: string the name of the histogram
 */

void H2D_new_histogram(Mc_hist2D *histogram, double max, double min, double bin_size, int n_max,
									char *name, int restart);
/*FUNCTION: destroy: deallocate the histogram*/
void H2D_destroy(Mc_hist2D *histogram);

/*Function: add value: adds a single value to the histogram
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 * input num: the value of the number
 */
void H2D_add_value(Mc_hist2D *histogram, double var, int num);

/*FUNCTION: print: prints the histogram to a file called NAME.dat
 *
 *input : the histogram to be printed
 *input volume: the volume of the system (or area in 2D)
 */
void H2D_print_histogram(Mc_hist2D *histogram, double volume);


#endif
