/*This is a module for building two dimensional histograms where the first dimension 
 * is just a number (like number of particles) and the second is a scalar variable (such as energy)
 *
 * The module prefix if H2D_
 */


#include"std_h.h"
#include"histograms2D_h.h"

/*
 * FUNCTION: Constructor: initialise a new histogram
 *
 * double var_max: the maximum value of the variable
 * double var_min: the minimum value of the variable
 * double bin_size: the (maximum) size of the bins for the variable
 * int n_max: the maximum value of the number
 * name: string the name of the histogram
 */

void H2D_new_histogram(Mc_hist2D *histogram, double max, double min, double bin_size, int n_max,
									char *name, int restart) {

	histogram->n_bins = (int) (max-min)/bin_size;
	histogram->bin_size = (max-min)/histogram->n_bins;
	histogram->var_max = max;
	histogram->var_min = min;
	histogram->n_max = n_max;
	histogram->freq = malloc(sizeof(long)*histogram->n_max*histogram->n_bins);
	strcpy(histogram->name,name);

	int i,j;
	for(i = 0; i < histogram->n_max*histogram->n_bins; i++) {
		histogram->freq[i] = 0;
	}

	if (restart) {
		char command[50];
		sprintf(command,"%s.dat",histogram->name);
		FILE *fp;
		char *line = NULL;
		size_t len = 0;
		ssize_t read;
		fp = fopen(command,"r");
		int first = 1;
		long val;
		
		while ((read = getline(&line, &len,fp)) != -1) {

			if (first) {
				first = 0;
			} else {
				sscanf(line,"%d %d %ld %*[\n]",&i,&j,&val);
				histogram->freq[i*histogram->n_max+j] = val;	
			}
		}
				

		fclose(fp);		
	}
}

/*FUNCTION: destroy: deallocate the histogram*/
void H2D_destroy(Mc_hist2D *histogram) {
	free(histogram->freq);
}

/*Function: add value: adds a single value to the histogram
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 * input num: the value of the number
 */
void H2D_add_value(Mc_hist2D *histogram, double var, int num) {
	double place = histogram->var_min + histogram->bin_size;
	int i;
	for (i = 0; i < histogram->n_bins; i++) {
		if (var < place) {
			histogram->freq[num*histogram->n_max + i]++;
			break;
		} else {
			place += histogram->bin_size;
		}
	}
}

/*FUNCTION: print: prints the histogram to a file called NAME.dat
 *
 *input : the histogram to be printed
 */
void H2D_print_histogram(Mc_hist2D *histogram, double volume) {

	char command[50];
	FILE *fp;
	int i,j;
	
	sprintf(command,"%s.dat",histogram->name);
	fp = fopen(command,"w");
	fprintf(fp,"#volume %lf\n",volume);
	for (i = 0; i < histogram->n_max; i++) {
		for(j = 0; j < histogram->n_bins; j++) {
			if (histogram->freq[i*histogram->n_max+j]) {
				fprintf(fp,"%d %d %ld\n",i,j,histogram->freq[i*histogram->n_max+j]);
			}
		}
	}
	fclose(fp);
}

	
