/*This is a module for collecting and printing 1D histograms of simple scalar variables.
 *
 * THe module prefix is HG_
 **/


#include"std_h.h"
#include"histogramsRE_h.h"
#include"mpi_h.h"

/*
 *FUNCTION : constructor: make an new histogram
 *
 *params histogram: a struct for the histogram being initialised
 *
 *int max: the maximum value of the variable
* int min: the minimum value of the variable
 *double bin_size: the (maximum) size of the bins (will be changed slightly)
 *string name: the name of the histogram
 */

void HG_new_histogram(Mc_histogram *histogram, double max, double min, double bin_size, char *name,
			int restart) {

	histogram->n_bins = (int) (max-min)/bin_size;
	histogram->bin_size = (max-min)/histogram->n_bins;
	histogram->max = max;
	histogram->min = min;
	histogram->freq = malloc(sizeof(long)*histogram->n_bins);
	strcpy(histogram->name,name);
	int i;
	if (!restart) {
		for (i = 0; i < histogram->n_bins; i++) {
			histogram->freq[i] = 0;
		}
	} else {
		char command[50];
		sprintf(command,"%s_histogram.csv",histogram->name);
		FILE *fp;
		fp = fopen(command,"r");
		for (i = 0; i < histogram->n_bins; i++) {
			fscanf(fp,"%*f %*c %ld %*[\n]",(histogram->freq)+i);
		}
		fclose(fp);
	}
}

/*FUNCTION: destroy: deallocate the histogram*/
void HG_destroy(Mc_histogram *histogram) {
	free(histogram->freq);
}

/*FUNCTION: Add value: adds a single value to the histogram
 *
 * Param histogram: the histogram to be added to
 * Input value: the value to be added
 */

void HG_add_value(Mc_histogram *histogram, double value) {

	double place  = histogram->min + histogram->bin_size;
	int i;
	for(i = 0; i < histogram->n_bins; i++) {
		if (value < place) {
			histogram->freq[i] ++;
			break;
		} else {
			place += histogram->bin_size;
		}
		
	}
}

/*FUNCTION: Add array: adds all the values in an array to the histogram
 *
 *params: histogram: the histogram to be added to
 *input double*: and array of values to be added
 *input num: the number of values in the array
 */

void HG_add_array(Mc_histogram *histogram, double *values, int num) {

	double place;
	int i,j;
	for (i = 0; i < num; i++) {
		place = histogram->min + histogram->bin_size;
		for (j = 0; j < histogram->n_bins; j++) {
			if (values[i] < place) {
				histogram->freq[j]++;
				break;
			} else {
				place += histogram->bin_size;
			}
		}
	}
}

/*FUNCTION: print: prints the histogram to a file called NAME_histogram.csv
*
*param histogram: the histogram to be printed
*/

void HG_print_histogram (Mc_histogram *histogram) {
	int i;
	FILE *fp;
	char command[50];
	sprintf(command,"%s_histogram.csv",histogram->name);
	fp = fopen(command,"w");
	for (i = 0; i < histogram->n_bins; i++) {
		fprintf(fp,"%f,%ld\n", histogram->min + (i*histogram->bin_size)
						      +  0.5*(histogram->bin_size), 
					histogram->freq[i]);
	}
	fclose(fp);
}


/*FUNCTION :sort: sorts the histograms from each mpi repetition into the correct ranks
 *
 * final_histogram: an initialised histogtam to have the data sorted into
 * histograms: a list of the histgrams for each index
 * size: the total number of MPI ranks
 *
 **/

void HG_sort_histograms (Mc_histogram *final_histogram,Mc_histogram *histograms, int size) {

	int i;
	int n_bins = histograms[0].n_bins;
	long *temp_freq;
	long *new_freq;
	new_freq = malloc (sizeof(long)*n_bins);
	for (i = 0; i < size; i++) {
		temp_freq = histograms[i].freq;
		MPI_Reduce(temp_freq, new_freq, n_bins, MPI_LONG, MPI_SUM, i, MPI_COMM_WORLD);
	}
	free(final_histogram->freq);
	final_histogram->freq = new_freq;
}






