#ifndef HISTOGRAMSRE_H
#define HISTOGRAMSRE_H

/*This is a module for collecting and printing 1D histograms of simple scalar variables.
 *
 * THe module prefix is HG_
 **/

typedef struct hgmcstrc {

	double bin_size;
	double max;
	double min;
	int n_bins;
	long *freq;
	char name[50];

} Mc_histogram;

/*
 *FUNCTION : constructor: make an new histogram
 *
 *params histogram: a struct for the histogram being initialised
 *
 *int max: the maximum value of the variable
* int min: the minimum value of the variable
 *double bin_size: the (maximum) size of the bins (will be changed slightly)
 *string name: the name of the histogram
 */

void HG_new_histogram(Mc_histogram *histogram, double max, double min, double bin_size, char *name,
			int restart);

/*FUNCTION: destroy: deallocate the histogram*/
void HG_destroy(Mc_histogram *histogram);

/*FUNCTION: Add value: adds a single value to the histogram
 *
 * Param histogram: the histogram to be added to
 * Input value: the value to be added
 */

void HG_add_value(Mc_histogram *histogram, double value);

/*FUNCTION: Add array: adds all the values in an array to the histogram
 *
 *params: histogram: the histogram to be added to
 *input double*: and array of values to be added
 *input num: the number of values in the array
 */

void HG_add_array(Mc_histogram *histogram, double *values, int num);

/*FUNCTION: print: prints the histogram to a file called NAME_histogram.csv
*
*param histogram: the histogram to be printed
*/

void HG_print_histogram (Mc_histogram *histogram);

/*FUNCTION :sort: sorts the histograms from each mpi repetition into the correct ranks
 *
 * final_histogram: an initialised histogtam to have the data sorted into
 * histograms: a list of the histgrams for each index
 * size: the total number of MPI ranks
 *
 **/

void HG_sort_histograms (Mc_histogram *final_histogram,Mc_histogram *histograms, int size);

#endif
