
/* a module for implementing the truncated, shifted and smoothed Lnnard-Jones potential
 *
 * The prefix is ldts_
 *
 * 
 */

#include"std_h.h"
#include"mc_math_h.h"
#include"lennard_jones_ts_h.h"

/*FUNCTION: get params: A helper function to set up the params array for this function to be
 * passed. Not required but useful, as it does all the differentiating for you etc.
 *
 * Output: double params[] (array of size 5)
 *		params[0] = epsilon, the inverse temperature in reduced units
 *		params[1] = The cutoff distance for the potential. The unit of distance is to
 *			the distance at which the Lennard Jones potential
 *			has it's minimum.
 *		params[2] = the value of the Lennard jones potential at the cutoff
 *		params[3] = the value of the differential of the potential
 *				at the cutoff
 *		params[4] = the inverse magnitude of the potential at r = 1, eps = 1
 *
 * Input: double epsilon: The value of the inverse temperature
 * input double cutoff: the cutoff length. 	
 */

void ldts_get_params(double params[],double epsilon,double cutoff) {
	params[0] = epsilon;
	params[1] = cutoff;
	double r6 = 1/(pow(cutoff,6));
	params[2] = epsilon*(r6*r6 - 2*r6);
	params[3] = -12*epsilon*(pow(cutoff,-13) - pow(cutoff,-7));
	params[4] = (1-params[2] + (1-params[1])*params[3]);	
}

/*
 FUNCTION: double lennard_jones2: calculates the pairwise ld potential
 * OUTPUT: ld potential in units of kT
 * INPUT: double distance2 the SQUARE of the separation of the two particles
 */

double ldts_potential2(double distance2, double params[]) {
	if (distance2 > params[1]*params[1]) {
		return 0;
	} else {
		double r6 = 1/(distance2*distance2*distance2);
		return params[0]*(r6*r6 - 2*r6) - params[2] -
                                    (sqrt(distance2)-params[1])*params[3];
	}

}

/*
 FUNCTION: double lennard_jones: calculates the pairwise ld potential
 * OUTPUT: ld potential in units of kT
 * INPUT: double distance the separation of the two particles
 */

double ldts_potential(double distance, double params[]) {

	if (distance > params[1]) {
		return 0.0;
	} else {
		double r6 = 1/(distance*distance*distance*distance*distance*distance);
		return params[0]*(r6*r6 - 2*r6) - params[2] -
                                    (distance-params[1])*params[3];
	}


}
