#ifndef NEIGHBOURS_H
#define NEIGHBOURS_H

/*a minimodule to deal with the nearest neighbours of a system of particles#
 *
 * the prefix is NEI_
 */

/*function: neighbours calculate: gets the number of nearest neighbours of 
 *all the particles in a state
 *
 *Return: pointer to a list of neighbours
 *input: state: the current state of the system
 *input *param: a param (not used)
 *input cutoff: the proximity of a nearest neighbour
 */

int * NEI_int_calculate (Mc_state *state, double *param, double cutoff, int periodic);

/*function: double neighbours: a version of neighbours calculate that returns doubles rather than
 * ints
 *
 */
double * NEI_double (Mc_state *state, double *param, double cutoff, int periodic);

/*function: gets the number of particles in the liquid like state
 * return number of liquid particles
  *input: state: the current state of the system
  *input d_cutoff: the proximity of a nearest neighbour
  *input n_cutoff: the required number of neighbours
*/
double NEI_liquid(Mc_state *state, double *param, double d_cutoff, double n_cutoff, int periodic);

#endif
