/*
 * Function READ_options: reads in the file input and populates
 * it with the keywords. It does some validation checks and so on as well.
 */

#include"std_h.h"
#include"mc_math_h.h"
#include"keywords_h.h"
#include"geometry_h.h"
#include"lennard_jones_ts_h.h"
#include"morse_ts_h.h"
#include"dipole_ts_h.h"
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"cluster_moves_h.h"
#include"bond_order_6h.h"
#include"bond_order6_plane_h.h"
#include"bond_order6_torus_h.h"
#include"lennard_jones_ts_lambda_h.h"
#include"read_h.h"

void READ_options(Data *input, FILE *options) {

	char command[50];
	char error[50];

	/*defaults*/
	strcpy(input->geometry, "SPHERE");
	input->radius = -1.0;
	input->delta_r = 0.5;
	input->deltaC_move = 0.0;
	input->deltaC_spin = 0.0;
	input->move = GEO_mv_sphere;
	input->random = GEO_random_sphere;	

	strcpy(input->potential_name,"LENNARDJONESTS");
	input->epsilon = -1.0;
	input->potential_cutoff = -1.0;
	input->rho = -1.0;
	input->potential = ldts_potential;

	input->activity = 0.0;

	input->cell_box_edge = -1.0;
	input->cell_min_size = -1.0;
	
	input->neighbour_cutoff = 1.3;
	input->seed = 0;

	input->n_particles_initial = -1;
	input->read_file = 0;
	input->restart = 0;

	input->max_steps = -1;
	input->equilibriate = -1;
	input->record_rate = -1;
	input->average_rate = -1;

	input->lambda = 0.0;
	input->n_target = 1;
	input->umbrella_rate = -1;
	input->order_cutoff = 0;

	/*read in data*/
	if (options == NULL) die ("Could not open options file!");
	while (read_line(options)) {
		get_string(command,sizeof(command));
		upper_case(command);
		
		if (*command == '#') {
			continue;
		} else if (strcmp (command,"GEOMETRY") == 0) {
			if (!get_string(input->geometry,sizeof(input->geometry)))
								die("could not get geometry");
		} else if (strcmp (command,"RADIUS")==0){
			if (!get_double(&(input->radius))) die ("Could not get radius");
		} else if (strcmp (command, "BOXX")==0) {
			if (!get_double(&(input->box[0]))) 
							die ("Could not get box x dimension");
		}else if (strcmp (command, "BOXY")==0) {
			if (!get_double(&(input->box[1]))) 
							die ("Couble not get box y dimension");	
		}else if (strcmp (command, "TORUSA")==0) {
			if (!get_double(&(input->torusA))) 
						die ("Couble not get torusA dimension");
		}else if (strcmp (command, "TORUSC")==0) {
			if (!get_double(&(input->torusC))) 
					die ("Couble not get torus c dimension");		
		} else if (strcmp (command,"POTENTIAL")==0) {
			if (!get_string(input->potential_name,sizeof(input->potential_name)))
								die("Could not get potential");
		} else if (strcmp (command, "EPSILON") == 0) {
			if (!get_double(&(input->epsilon))) die ("Could not get epsilon");
		} else if (strcmp (command, "LAMBDASCALE") == 0) {
			if (!get_double(&(input->lambda_scale))) die ("Could not get lambda_s");
		} else if (strcmp (command, "ALPHA") == 0) {
			if (!get_double(&(input->alpha))) die ("Could not get alpha");
		} else if (strcmp (command, "GAMMA") == 0) {
			if (!get_double(&(input->gamma))) die ("Could not get gamma");
		} else if (strcmp(command, "RHO") == 0) {
			if (!get_double(&(input->rho))) die ("could not get rho");
		} else if (strcmp (command, "CUTOFF") == 0) {
			if (!get_double(&(input->potential_cutoff))) die ("Could not get cutoff");
		} else if (strcmp (command, "INITFROMFILE") == 0) {
			if (!get_int(&(input->read_file))) die ("Could not get readFile");
		} else if (strcmp (command, "RESTART") == 0) {
			if (!get_int(&(input->restart))) die ("Could not get restart");
		} else if (strcmp(command,"ACTIVITY") == 0) {
			if (!get_double(&(input->activity))) die ("Could not get activity");
		}else if (strcmp(command,"NEIGHBOURCUTOFF") == 0) {
			if (!get_double(&(input->neighbour_cutoff))) die ("Could not get activity");
		} else if (strcmp(command,"NUMBEROFPARTICLES") == 0) {
			if (!get_int(&(input->n_particles_initial))) 
							die ("Could not get particle number");
		} else if (strcmp(command,"SEED") == 0) {
			if (!get_int(&(input->seed))) die ("could not get random seed");
		} else if (strcmp(command,"MAXSTEPS")==0) {
			if (!get_int(&(input->max_steps))) die ("Could not get maximum steps");
		} else if (strcmp(command,"EQUILIBRIATE") == 0) {
			if (!get_int(&(input->equilibriate))) die ("could not get equlibriation");
		} else if (strcmp(command,"FRAMERATE") ==0) {
			if (!get_int(&(input->frame_rate))) die ("Could not get frame rate");
		} else if (strcmp(command,"RECORDRATE")==0) {
			if (!get_int(&(input->record_rate))) die ("Could not get record rate");
		} else if (strcmp(command,"AVERAGERATE")==0) {
			if(!get_int(&(input->average_rate))) die ("Could not get average rate");
		} else if (strcmp(command,"LAMBDA") == 0) {
			if(!get_double(&(input->lambda))) die ("Could not get lambda");
		} else if (strcmp(command,"TARGETNUCLEUS")==0) {
			if(!get_int(&(input->n_target))) die ("Could not get target nucleus");
		} else if (strcmp (command,"MAXNUCLEUS")==0) {
			if(!get_int(&(input->n_max))) die ("Could not get max nucleus");
		} else if (strcmp(command,"UMBRELLARATE")==0) {
			if(!get_int(&(input->umbrella_rate))) die ("Could not get umbrella rate");
		} else if (strcmp(command,"ORDERCUTOFF")==0) {
			if (!get_double(&(input->order_cutoff))) die ("Could not get order cutoff");
		}
	}

	/*get derived values*/
	input->cell_min_size = input->potential_cutoff;
	if (input->umbrella_rate == -1) input->umbrella_rate = input->average_rate;

	upper_case(input->geometry);
	if (strcmp(input->geometry,"SPHERE")==0) {
		input->move = GEO_mv_sphere;
		input->random = GEO_random_sphere;
		input->move_cluster = CMV_move_cluster_sphere;
		input->spin_cluster = CMV_spin_cluster_sphere;
		input->get_area = GEO_area_sphere;
		input->periodic = 0;
		input->delta_r = 0.5;
		input->deltaC_move = M_PI/100;
		input->deltaC_spin = M_PI/100;
		input->cell_box_edge = 2*(input->radius)+1.0;
		input->paramsG = malloc(sizeof(double)*1);
		input->paramsG[0] = input->radius;
		input->BO6_calculate = BO6_sphere_calculate;

	} else if (strcmp(input->geometry,"PLANE")==0){
		input->move = GEO_mv_plane;
		input->random = GEO_random_plane;
		input->move_cluster = CMV_move_cluster_plane;
		input->spin_cluster = CMV_spin_cluster_plane;
		input->get_area = GEO_area_plane;
		input->periodic = 1;
		input->delta_r = 0.5;
		input->deltaC_move = 0.5;
		input->deltaC_spin = M_PI/100;
		input->cell_box_edge = (input->box[0])+0.0001;
		input->paramsG = malloc(sizeof(double)*2);
		input->paramsG[0] = input->box[0];
		input->paramsG[1] = input->box[1];
		input->BO6_calculate = BO6_plane_calculate;

	} else if (strcmp(input->geometry,"TORUS")==0){
		input->move = GEO_mv_torus;
		input->random = GEO_random_torus;
		input->get_area = GEO_area_torus;
		input->periodic = 0;
		input->delta_r = 0.06;
		input->cell_box_edge = 2*(input->torusA + input->torusC)+5.0;
		input->paramsG = malloc(sizeof(double)*2);
		input->paramsG[0] = input->torusA;
		input->paramsG[1] = input->torusC;
		input->BO6_calculate = BO6_torus_calculate;
	} else {
		die ("Geometry not recognised");
	}

	upper_case(input->potential_name);
	if (strcmp(input->potential_name,"LENNARDJONESTS")==0) {
		input->potential = ldts_potential;
		input->paramsP = malloc(5*sizeof(double));
		ldts_get_params(input->paramsP, input->epsilon, input->potential_cutoff);
	} else if (strcmp(input->potential_name,"LENNARDJONESLAMBDA")==0) {
		input->potential = ldtsl_potential;
		input->paramsP = malloc(9*sizeof(double));
		ldtsl_get_params(input->paramsP,input->epsilon,input->potential_cutoff,
							input->lambda_scale,input->alpha);
		
	} else if (strcmp(input->potential_name,"MORSETS")==0) {
		input->potential =  mts_potential;
		input->paramsP = malloc (5*sizeof(double));
		mts_get_params(input->paramsP, input->epsilon, input->rho, input->potential_cutoff);
	} else if (strcmp(input->potential_name,"DIPOLE")==0) {
		input->cell_min_size = input->cell_box_edge-1.0;
		//input->potential = dp_potential;
		input->paramsP = malloc(1*sizeof(double));
		//dp_get_params(input->paramsP,input->gamma,
                              //input->n_particles_initial,input->get_area(input->paramsG));
	} else {
		die ("Potential not recognised");
	}

	/*validation checks*/
	if (input->radius < 0.0) die ("Please define a positive radius");
	if (input->radius < 1.0) die ("The manifold must be larger than the surface");
	if (input->radius < input->delta_r) 
				die ("The particles must not move greater than the radius");
	if (input->delta_r < 0.0) die ("please define a positive delta_r");
	if (input->epsilon < 0.0) die ("Please defien a positive epsilon");
	if (input->potential_cutoff < 0.0) die ("Please define a positive potential cutoff");
	if (input->activity < -0.0000000001) die ("please define a positive activity");
	if (input->n_particles_initial < 0) die ("Please pick a positive number of particles");
	if (input->read_file != 1 && input->read_file !=0 ) die ("ReadFile should be either 0 or 1");
	if (input->max_steps < 0) die ("please pick a positive maxSteps");
	if (input->equilibriate < 0) die ("please pick a positive or 0 equilibriation time");
	if (input->record_rate < 0) die ("please pick a positive or 0 recordRate");
	if (input->record_rate % input->average_rate != 0 ) 
					die ("record_rate must be a multiple of average rate");
	if (input->max_steps % input->average_rate != 0) 
						die ("MaxSteps must be a multiple of AverageRate");
	if (input->average_rate < 0) die ("please pick a positive averageRate");
	if (input->max_steps % input->average_rate != 0 )
						die("Maxsteps must be a multiple of AveragegRate");
	if (input->umbrella_rate && input->average_rate % input->umbrella_rate != 0)
					die("average rate must be a multiple of umbrella rate");
	if (input->restart && input->read_file) 
				die("Cannot initialise from file and restart from old simulation");
	if (input->restart && input->equilibriate) 
		die("You should not equilibriate after a restart. To restart an equilibriation, copy out the stateXX.csv file and use InitFromFile");
	

}

/*deallocate a read struct*/
void READ_destroy(Data *input) {
	free(input->paramsP);

}
