/*................................................................................

weight2D.c

Weights a 2D "density of states" for sticky spheres to a new value of tau and
a new activity to give the 1D density histogram.  Works only with the output
from version 5 of hist.c

MM, 12.vii.02

................................................................................*/
 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{
   char line[200];
   long i;
   long mm, nn;
   long maxn;
   double activity;
   double *dhist;
   double norm;
   double rho;
   double tau;
   double volume;
   double x;
   FILE *fh;

   if (argc < 4) {
      printf ("Usage: %s <new tau> <new activity> <log_omegaNM filename>,\n", argv[0]);
      exit (1);
   }

   tau = atof(argv[1]);
   activity = atof(argv[2]);

   /* Determine volume of system and hence maximum values of N and M from first histogram */
   fh = fopen(argv[3], "r");
   if (fh == NULL) {
      fprintf (stderr, "ERROR: Could not open %s\n", argv[3]);
      exit (1);
   }
   volume = -1.0;
   while (fgets(line, sizeof(line), fh) != NULL) {
      sscanf(line, "N %ld %le", &nn, &rho);
      if (line[0] == 'N' && nn > 0) {
         volume = nn/rho;
         break;
      }
   }
   fclose (fh);
   if (volume < 0.0) {
      fprintf (stderr, "ERROR: Could not determine system volume from %s\n", argv[3]);
      exit (1);
   }
   maxn = 1000;

   /* Accumulate unnormalised 1D density histogram */
   fh = fopen(argv[3], "r");
   dhist = (double *)malloc( maxn*sizeof(double) );
   for (i=0; i<maxn; i++) dhist[i] = 0.0;
   while (fgets(line, sizeof(line), fh) != NULL) {
      sscanf(line, "%*c %ld %le", &i, &x);
      if (line[0] == 'N') {
         nn = i;
      } else if (line[0] == 'M') {
         dhist[nn] += exp(x + nn*log(activity) + (3.0*i+1.5)*tau);
	 printf("%f, %ld, %f, %f\n",x,nn,activity,tau);
      } else {
         fprintf (stderr, "ERROR: Line beginning with %c in %s\n", line[0], argv[3]);
         exit (1);
      }
   }
   fclose (fh);

   /* Normalise histogram in the density */
   norm = 0.0;
   for (nn=0; nn<maxn; nn++) norm += dhist[nn];
   norm = 1.0/ norm;
	printf("NORM: %f\n",norm);
   /* Write out normalised histogram */
   fh = fopen("newhist.dat", "w");
   for (nn=0; nn<maxn; nn++) {
      fprintf (fh, "%.8lf  %.8le  %ld\n", (0.25*M_PI*nn)/volume, dhist[nn]*norm, nn);
   }
   fclose (fh);
   printf ("Histogram at tau = %lf, activity = %le written to newhist.dat\n", tau, activity);
}
