#ifndef ENERGY_H
#define ENERGY_H

/* This module contains functions for calculating the potential energy felt
 * by various sets of particles. Only pairwise interactions may be handled.
 * The form of the potential if defined by the global function pointers found
 * in potential.c

 *the private prefeix for this module is e_
 */

#include"std_h.h"
#include"twister.c"
#include"potential_h.h"
#include"geometry_h.h"
#include"twister.c"
#include"n_particles.h"

/* global*/
/*contains the imaginary temperature for the bond making*/
double temp;

/*setters*/
void set_max_temp (double u);
void set_min_temp (double u);
void set_epsilon(double u);

/*getter*/
double get_epsilon();

/* read function, just passes to P_read and g_read*/
void e_read (FILE *infile);

/*FUNCTION: get_temp sets the imaginary temperature uniformly ditributed
 * between max_temp and min_temp
 *
 */
void get_temp();

/*FUNCTION: energy_calc gets the potential energy of a single particle with
 * generalised coordinates u,v. The particle may be real (as in, existing in
 * the system) or imaginary (a test particle)
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 */

double energy_calc(double coord_u[],double coord_v[], double a_u, double a_v, int index) ;

/*FUNCTION: energy_calc_DA is used to calculate the (augmented,not real) energy of
 * a particle at u ,v. Using the data augmentation scheme, only particles in
 * different clusters are considered, and the imaginary temperature "temp" is used
 * to calculate the imaginary "bond energy"
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double temp: the imaginary temperature used to calculate the "energy" of
 *	the bond matrix
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 *	int cluster_of[]: A list containing an index which uniquely identifies 
 *	the sluster each particle belongs to.
 */

double energy_calc_DA (double coord_u[], double coord_v[], double temp, int target, int cluster_of[]);

/*FUNCTION: energy_total is used to calculate the  energy of
 * the entire system in units of epsilon/kt
 *
 * Output: type double: The energy
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 */

double energy_total(double coord_u[], double coord_v[]);

/*
 *FUNCTION type int overlap:determines whether two particles overlap at all
 * Output: int: zero if no overlap, otherwise some non zero int
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double temp: the imaginary temperature used to calculate the "energy" of
 *	the bond matrix
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 */

int overlap (double coord_u[], double coord_v[], int target);

#include"energy.c"
#endif
