#ifndef FLAT_H
#define FLAT_H

/*a module containing the basic geometrical functions on a rectangular flat plane
 * u: the x coordinate/x_max
 * v: the v coordinate/y_max
 *
 *the prefix for this module if fl_
 */

#include"std_h.h"
#include"mc_math_h.h"
#include"keywords.c"

/*read function for the plane*/
void fl_read (FILE *infile);

/*FUNCTION: cartesian is a function which converts from the current
 * to cartesians.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void fl_cartesian(double *x, double *y, double *z,double coord_u, double coord_v);

/*FUNCTION: cartesian_bx is a function which converts from the current
 * to cartesians, but shifts the origin so that no coordinates are negative.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void fl_cartesian_bx(double *x, double *y, double *z,double coord_u, double coord_v);

/* FUNCTION: distance_NI returns the cartesian distance between two
 * particles. Under periodic bounderies, the nearest image convention is used.
 *
 * Output: double, The particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double fl_distance_NI (double u1,double v1, double u2, double v2);

/* FUNCTION: distance_NI_2 returns the square of the cartesian distance
 * between two particles. Faster than sqrt(distance_NI()).
 *
 * Output: double, The square of the particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double fl_distance_NI_square(double u1,double v1, double u2, double v2);

/* FUNCTION: uv_coords converts cartesian coordinates to generalised coordinates
 *
 * Input: double x,y,z the cartsian coordinates of the particle
 * Output: double *u,*v, pointers to a location to contain the generalised coordinates u,v
 */
void fl_uv_coords(double *u, double *v, double x, double y, double z);

#include"flat.c"
#endif
