#define FINALIZE_CALLED

#include"mc_init_h.h"
#include"monte_carlo_PT_h.h"
#include"mpi_h.h"
#include"pt_init_h.h"
#include"std_h.h"
#include"n_particles.h"
#include"twister.c"



int main(void){

pt_init();

//read in all the inputs 
char command [50];
FILE *fp;
sprintf(command,"options%d.txt",pt_rank);
fp = fopen(command,"r");
mcpt_read(fp);
fclose(fp);

//set n_particles to start with

		/*----------------define all the variables, lists etc------------------*/


//a variable to store the current time in-keeps track of the simulation
time_t curTime;

//a list of the u coordinate of each particle
double *coord_u;

//a list of the v coordinate of each particle
double *coord_v;

//allocate the lists
coord_u = (double *)malloc(sizeof(double)*n_particles);
coord_v = (double *)malloc(sizeof(double)*n_particles);

//ints for recording the number of successful and failed moves
int accept_ratio_m;
int total_moves_m;
int accept_ratio_s;
int total_moves_s;

//dummies
long steps, i,j,k,n;
double rt;

		/*----------------------get inputs and set up rng--------------------*/

//seed the random number generator, trigger it once to be sure
seedMT(time(NULL));
randomF();

/*set up initial conditions and simulation length*/
sprintf(command,"options%d.txt",pt_rank);
fp = fopen(command,"r");
init_read(fp);
initialise(&coord_u,&coord_v);
fclose(fp);


//change directories
 sprintf(command,"mkdir run%d",pt_rank);
 system(command);
 sprintf(command, "cp options%d.txt run%d",pt_rank,pt_rank);
 sprintf(command,"run%d",pt_rank);
 chdir(command);

/*-------------------------------------equilibiration loop 1----------------------------------*/

	/* 	the first "monte carlo" loop. During this stage of the simulation
	 *	the simulation tracks the number of accepted moves and tunes the move lengths
	 *	until the acceptance ratio is about 0.25 for all move types. Therefore, detailed 		 *	balance does not hold during this section of the simulation.
	 */

steps = 0;
accept_ratio_s = 0;
total_moves_s = 0;

while (steps < equilibriate) {

	accept_ratio_m= 0;
	total_moves_m = 0;
	accept_ratio_s = 0;
	total_moves_s = 0;
	
	for(i = 0; i < 5000; i++) {

		//pick move type
		rt = randomF();

		//50% move 25% add 25% sub
		if (rt < 0.5) {
			total_moves_m++;
			if (particle_move(coord_u,coord_v)) accept_ratio_m++;
		} else if (rt <0.75) {
			particle_add(&coord_u,&coord_v);
		} else {
			particle_sub(&coord_u,&coord_v);
		}

		if (i%500 == 0) {
			//do an exchange
			if (i%1000 == 0) {
				if (pt_rank % 2 == 0 && pt_rank != 0) {
				  total_moves_s++;
				  if( exchange_down(coord_u,coord_v) ) accept_ratio_s++;
				} else if (pt_rank % 2 == 1 && pt_rank != pt_size-1) {
				  total_moves_s++;
				  if (exchange_up(coord_u,coord_v) ) accept_ratio_s++;
				}
			} else  {
				if (pt_rank % 2 == 0 && pt_rank != pt_size-1) {
				  total_moves_s++;
				  if( exchange_up(coord_u,coord_v) ) accept_ratio_s++;
				} else if (pt_rank %2 == 1 && pt_rank != 0) {
				  total_moves_s++;
				  if (exchange_down(coord_u,coord_v) ) accept_ratio_s++;
				}
			}
		}


	}

	//increase steps
	steps += 5000;

	//tune step length
	if ((float) accept_ratio_m/total_moves_m > 0.25) {
		if (delta_u < max_u) delta_u *= 1.05;
		if (delta_v < max_v) delta_v *= 1.05;
	} else if ((float) accept_ratio_m/total_moves_m < 0.25) {
		if (delta_u > min_u) delta_u /= 1.05;
		if (delta_v > min_v) delta_v /= 1.05;
	}

	save(coord_u,coord_v);	

}


/*-------------------------------------SIMULATION LOOP ----------------------------------*/

	/* 	This is the real simulation, it is identical to equilibriation loop 2, but every 
	 *	Record_Rate sweeps it takes measurments of the system and records the results in a 
	 *	number of data files. Simulations that are restarted from an old eq configuration
	 *	go straight here. 
	 */

steps = 0;
int count_records = 0;

/*----allocate memory for data---*/
int *n_list = malloc(sizeof(int)* max_steps/record_rate);
int *i_list = malloc(sizeof(int)* max_steps/record_rate);
double *en_list = malloc(sizeof(double) * max_steps/record_rate);


while (steps < max_steps) {

	accept_ratio_m= 0;
	total_moves_m = 0;
	accept_ratio_s = 0;
	total_moves_s = 0;
	
	for(i = 0; i < record_rate; i++) {

		//pick move type
		rt = randomF();

		//50% move 25% add 25% sub
		if (rt < 0.5) {
			particle_move(coord_u,coord_v);
		} else if (rt <0.75) {
			particle_add(&coord_u,&coord_v);
		} else {
			particle_sub(&coord_u,&coord_v);
		}

		if (i%500 == 0) {
			//do an exchange
			if (i%1000 == 0) {
				if (pt_rank % 2 == 0 && pt_rank != 0) {
					exchange_down(coord_u,coord_v);
				} else if (pt_rank % 2 == 1 && pt_rank != pt_size-1) {
					exchange_up(coord_u,coord_v);
				}
			} else  {
				if (pt_rank % 2 == 0 && pt_rank != pt_size-1) {
					exchange_up(coord_u,coord_v);
				} else if (pt_rank %2 == 1 && pt_rank != 0) {
					exchange_down(coord_u,coord_v);
				}
			}
		}


	}

	steps += record_rate;
	n_records++;
	save(coord_u,coord_v);
	n_list[count_records] = n_particles;
	en_list[count_records] = energy_total(coord_u,coord_v);
	i_list[count_records] = i_temperature;
	count_records++;
	save(coord_u,coord_v);

	//just in case rounding errors//
	if (count_records > (max_steps/record_rate) - 1) break;	

}

/*---------------------------------------SAVE THE DATA---------------------*/
fp = fopen("num_energy.csv","a");

for (i = 0; i < count_records; i++) {
	fprintf(fp,"%d,%d,%lf\n",i_list[i],n_list[i],en_list[i]);
}

fclose(fp);

free(coord_u);
free(coord_v);
free(n_list);
free(en_list);
free(i_list);



pt_end();
}
