#ifndef GEOMETRY_H
#define GEOMETRY_H

/*a module which holds othe modules anf functions for doing geometry on a general
 * interface. Converts betwwen coordinate systems etc.
 *
 * the prefix is g_
 */
#include"sphere_h.h"
#include"flat_h.h"
#include"keywords.c"
#include"std_h.h"

/*read function for geometry*/
void g_read (FILE *infile);

/*________globals_________*/

/*the maximum value of the general coordinate u*/
double max_u;
/*the minimum value of the general coordinate u*/
double min_u;
/*contains the boundry type of coordinate u. the value is 0 if the boundry is (inherently) periodic ie. phi in spherical polars. The value is 1 if the boundry is reflective*/
int bound_u;
/*the maximum value of the general coordinate v*/
double max_v;
/*the minimum value of the general coordinate v*/
double min_v;
/*contains the boundry type of coordinate v. the value is 0 if the boundry is (inherently) periodic ie. phi in spherical polars. The value is 1 if the boundry is reflective*/
int bound_v;

/*FUNCTION POINTER: cartesian is a function which converts from the current
 * to cartesians.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */
void (*cartesian) (double *x, double *y, double *z, double coord_u,double coord_v);

/*FUNCTION POINTER: cartesian_bx is a function which converts from the current
 * to cartesians, but shifts the origin so that no coordinates are negative.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */
void (*cartesian_bx) (double *x, double *y, double *z, double coord_u,double coord_v);

/* FUNCTION POINTER: distance_NI returns the cartesian distance between two
 * particles. Under periodic bounderies, the nearest image convention is used.
 *
 * Output: double, The particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/
double (*distance_NI) (double u1, double v1, double u2, double v2);

/* FUNCTION POINTER: distance_NI_2 returns the square of the cartesian distance
 * between two particles. Faster than sqrt(distance_NI()).
 *
 * Output: double, The square of the particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/
double (*distance_NI_square) (double u1, double v1, double u2, double v2);

/* FUNCTION POINTER: uv_coords converts cartesian coordinates to generalised coordinates
 *
 * Input: double x,y,z the cartsian coordinates of the particle
 * Output: double *u,*v, pointers to a location to contain the generalised coordinates u,v
 */
void (*uv_coords) (double *u, double *v, double x, double y, double z);

#include"geometry.c"
#endif
