/*-----global-----*/
double ld_epsilon;
double ld_cuttoff = 1000;
double ld_at_cut = 0;
double d_ld_at_cut = 0;

/*read function*/
void ld_read (FILE *infile){

	char command [50];
	char error[20];
	
	int success_e = 0 ;
	int success_r =0;

	/*read in values*/

	if (infile == NULL) die ("Could not open options file!");
	rewind(infile);
	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"EPSILON")==0) {
			if (!get_double(&ld_epsilon)) die ("Could not read epsilon");
			success_e = 1;
		} else if (strcmp(command,"CUTTOFF")==0) {
			if (!get_double(&ld_cuttoff)) die ("Could not read cuttoff");
			ld_cuttoff = ld_cuttoff*ld_cuttoff;
			success_r = 1;
		}
	}

	if (success_e == 0) die ("epsilon not defined in options file");

	if (ld_epsilon < 0) {
		die("the well depth should be a positive number");
	}

	double r6 = 1/(ld_cuttoff*ld_cuttoff*ld_cuttoff);
	ld_at_cut = ld_epsilon*(r6*r6 - 2*r6);
	d_ld_at_cut = -12*ld_epsilon*(pow(sqrt(ld_cuttoff),-13) - pow(sqrt(ld_cuttoff),-7));

}


/*
 FUNCTION: double lennard jones: calculates the pairwise ld potential
 * OUTPUT: ld potential in units of kt
 * INPUT: the square of the separation of the two particles
 */
double lennard_jones(double distance_2) {
	if (distance_2 > ld_cuttoff) {
		return 0;
	} else {
		double r6 = 1/(distance_2*distance_2*distance_2);
		return ld_epsilon*(r6*r6 - 2*r6) - ld_at_cut -
                                    (sqrt(distance_2)-sqrt(ld_cuttoff))*d_ld_at_cut;
	}

}
