/*-----globals-----*/

double mc_activity;
int mc_record = 1;
int mc_record_energy = 0;

void mc_set_activity(double u) {
	mc_activity = u;
}
double mc_get_activity() {
	return mc_activity;
}

/*read file for monte carlo*/
void mc_read (FILE *infile) {
	//gc_read(infile);
	e_read(infile);
	m_read(infile);

	char command [50];
	char error[20];
	char record[10];
	char rec_en[10];
	
	mc_activity = 50 ;
	int success_n = 0;

	/*read in values*/

	if (infile == NULL) die ("Could not open options file!");
	rewind(infile);
	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"ACTIVITY")==0) {
			if (!get_double(&mc_activity)) die ("Could not read activity");
		} else if (strcmp(command,"NUMBEROFPARTICLES") == 0) {
			if (!get_int(&n_particles_initial)) die ("could not get number of particles");
			success_n = 1;
		} else if (strcmp(command,"RECORDSTATE") == 0) {
			if (!get_string(record,sizeof(record))) die ("could not get whether to keep records");
		} else if (strcmp(command,"RECORDENERGY") == 0) {
			if (!get_string(rec_en,sizeof(rec_en))) die ("could not get whether to keep records");
		}
	}

	if (strcmp(rec_en,"YES")==0) {
		mc_record_energy = 1;
	} else if (strcmp(rec_en,"NO")==0) {
		mc_record_energy = 0;
	}

	if (strcmp(record,"YES")==0) {
		mc_record = 1;
	} else if (strcmp(record,"NO")==0) {
		mc_record = 0;
	}
	

	if (success_n == 0) die ("number of particles not defined in options file");
	if (mc_activity < 0) {
		printf("%f\n",mc_activity);
		die("the activity should be positive");
	}
}

void save(double coord_u [],double coord_v []) {
	
	FILE *fp;
	int j;

		//record the final state for possible restarts//
	fp = fopen("final.csv","w");

	for (j=0;j<n_particles;j++) {
		fprintf(fp,"%f,%f\n",coord_u[j],coord_v[j]);
	}
	fclose(fp);

		//record some data//
	fp  = fopen("data.txt","w");
	fprintf(fp,"N_particles %ld\n",n_particles);
	fprintf(fp,"Delta_u %lf\n",delta_u);
	fprintf(fp,"Delta_v %lf\n",delta_v);
	fprintf(fp,"Records %ld\n",n_records);
	fclose(fp);


}

//records the current state in cartesians, does not overwrite, but appends
void record(double coord_u [],double coord_v []) {

	if (mc_record) {

		FILE *fp;
		int j;
		double x,y,z;

		//save the state of the system to file//
		fp = fopen("history.csv","a");
		fprintf(fp,"%ld,",n_particles);
		//printf("n_particles: %ld\n",n_particles);
	
		for (j=0;j<n_particles;j++) {
			cartesian(&x,&y,&z,coord_u[j],coord_v[j]);
			fprintf(fp,"%f,%f,%f,",x,y,z);
		}
		fprintf(fp,"\n");
		fclose(fp);
	}


}

//records the total energy of the system
void record_energy(double coord_u [],double coord_v []) {

	if (mc_record_energy) {
		FILE *fp2;
		int j;
		double energy = energy_total(coord_u,coord_v);
	
		fp2 = fopen("energy.csv","a");
		fprintf(fp2,"%f\n",energy);
		fclose(fp2);
	}
}

/*records the number of nearest neighbours for each particles*/
void record_neighbours (double coord_u[],double coord_v[]) {

	if (mc_record) {

		int i,j;

		int *neighbours = malloc(sizeof(int)*n_particles);
		for (i = 0; i < n_particles; i++) {
			neighbours[i] = 0;
		}
		double dist;
		FILE *fp;

		for (i = 0; i < n_particles-1; i++) {
			for (j = i+1;j< n_particles;j++) {
				dist = distance_NI_square (coord_u[i],coord_v[i],coord_u[j],coord_v[j]);
				if (dist < 2.0) {
					neighbours[i] ++;
					neighbours[j] ++;
				}
			}
		}

		fp = fopen("neighbours_history.csv","a");
		fprintf(fp,"%ld,",n_particles);
		for (i = 0; i < n_particles; i++) {
			fprintf(fp,"%d,",neighbours[i]);
		}
		fprintf(fp,"\n");
		fclose(fp);
		free(neighbours);
	}
}

/*
 * FUNCTION particle_move: this function takes a particle and performs a
 * single particle move monte carlo step.
 *
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 */

int particle_move (double coord_u[], double coord_v[]) {

	if (n_particles == 0) return 0;

	//select a particle to move
	int mover = randomMT() % n_particles;
	//get old state	
	double old_u = coord_u[mover];
	double old_v = coord_v[mover];
	double old_energy = energy_calc (coord_u, coord_v, coord_u[mover], coord_v[mover],mover);

	//move the particle
	int accept = move(coord_u+mover,coord_v+mover);
	if (accept == 0) return 0;
	
	//get new energy
	double energy = energy_calc (coord_u, coord_v, coord_u[mover], coord_v[mover],mover);
	//accept or reject
	if ( randomF() < exp(-1*(energy - old_energy)) ) { //accept
		return 1;
	} else {
		coord_u[mover] = old_u;
		coord_v[mover] = old_v;
		return 0;
	}
	return 0;
}

/*
 * FUNCTION particle_add: performs a grand canonical attempted particle addition
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int particle_add (double **coord_u, double **coord_v) {

	//generate a suggested position for the new particle
	double u_temp,v_temp;
	u_temp = randomF()*(max_u-min_u)+min_u;
	v_temp = randomF()*(max_v-min_v)+min_v;

	double *temptr;
	double *temptr2;

	//get energy
	double energy = energy_calc(*coord_u,*coord_v,u_temp,v_temp,n_particles+1);

	//accept/reject
	if (randomF() < (mc_activity/((float)n_particles+1.0))*exp(-1*energy) ) { //accept

		n_particles++;

		temptr = realloc(*coord_u,sizeof(double)*n_particles); 

		if (temptr != NULL) {
			*coord_u = temptr;
		} else {
			die("realloc failed in add");
		}

		temptr2 = realloc(*coord_v,sizeof(double)*n_particles);
		if (temptr2 != NULL) {
			*coord_v = temptr2;
		} else {
			die("realloc failed in add");
		}
		(*coord_u)[n_particles-1] = u_temp;
		(*coord_v)[n_particles-1] = v_temp;
		return 1;	
	} 
	return 0;
}

/*
 * FUNCTION particle_sub: performs a grand canonical attempted particle removal
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int particle_sub (double **coord_u, double **coord_v) {

	//check for special case
	if (n_particles == 0) return 0;
	double *temptr2;
	//select a particle to take away
	int mover = randomMT() % n_particles;

	//get energy
	double energy = energy_calc(*coord_u,*coord_v,(*coord_u)[mover],(*coord_v)[mover],mover);
	//accepr / reject 
	if (randomF() < (((float)n_particles)/mc_activity)*exp(energy)) {

		//accept
		
		//remove from lists
		int i;
		for (i = mover; i < n_particles-1; i++) {
			(*coord_u)[i] = (*coord_u)[i+1];
			(*coord_v)[i] = (*coord_v)[i+1];

		}

		//reallocate
		n_particles--;
		temptr2 = realloc(*coord_u,sizeof(double)*n_particles);
		if ( temptr2 ) {
			*coord_u = temptr2;
		} else if (n_particles == 0) {
			*coord_u = temptr2;
		} else {
			die("realloc failed in sub");
		}
		temptr2 = realloc(*coord_v,sizeof(double)*n_particles);
		if (temptr2) {
			*coord_v = temptr2;
		} else if (n_particles == 0) {
			*coord_v = temptr2;
		} else {
			die("realloc failed in sub");
		}

		return 1;
	}
	return 0;
}








