/*-----globals-----*/

double mc_activity;

int exchange_up_temp(double *coord_u, double *coord_v);
int exchange_down_temp (double *coord_u, double *coord_v);
int exchange_up_act(double *coord_u, double *coord_v);
int exchange_down_act (double *coord_u, double *coord_v);

/*read file for monte carlo*/
void mcpt_read (FILE *infile) {
	//gc_read(infile);
	e_read(infile);
	m_read(infile);

	char command [50];
	char error[20];
	char exchange[20];
	
	mc_activity = 0;
	int success_n = 0;
	i_temperature = pt_rank;
	i_activity = pt_rank;
	i_index_pt = &i_temperature;

	/*read in values*/

	if (infile == NULL) die ("Could not open options file!");
	rewind(infile);
	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"ACTIVITY")==0) {
			if (!get_double(&mc_activity)) die ("Could not read activity");
		} else if (strcmp(command,"NUMBEROFPARTICLES") == 0) {
			if (!get_int(&n_particles_initial)) die ("could not get number of particles");
			success_n = 1;
		} else if  (strcmp(command,"EXCHANGE") == 0) {
		  if (!get_string(exchange,sizeof(exchange))) die ("could not get exchange");
		}
	}


	upper_case(exchange);
	if (strcmp(exchange,"EPSILON")==0) {
		i_index_pt = &i_temperature;
		get_swap = p_get_epsilon;
		set_swap = p_set_epsilon;
		exchange_up = exchange_up_temp;
		exchange_down = exchange_down_temp;
	} else if (strcmp(exchange,"ACTIVITY") == 0) {
		i_index_pt = &i_activity;
		get_swap = mc_get_activity;
		set_swap = mc_set_activity;
		exchange_up = exchange_up_act;
		exchange_down = exchange_down_act;
	} else {
		die ("can not exchange that");
	}

	if (success_n == 0) die ("number of particles not defined in options file");
	if (mc_activity < 0) {
		printf("%f\n",mc_activity);
		die("the activity should be positive");
	}
}

void save_PT(double coord_u [],double coord_v []) {
	
	FILE *fp;
	int j;

		//record the final state for possible restarts//
	fp = fopen("final.csv","w");

	for (j=0;j<n_particles;j++) {
		fprintf(fp,"%f,%f\n",coord_u[j],coord_v[j]);
	}
	fclose(fp);

		//record some data//
	fp  = fopen("data.txt","w");
	fprintf(fp,"N_particles %ld\n",n_particles);
	fprintf(fp,"Delta_u %lf\n",delta_u);
	fprintf(fp,"Delta_v %lf\n",delta_v);
	fprintf(fp,"Records %ld\n",n_records);
	fprintf(fp,"Epsilon %f\n",get_epsilon());
	fprintf(fp,"i_swap %d\n",*i_index_pt);
	fclose(fp);


}

//records the current state in cartesians, does not overwrite, but appends
void record_PT(double coord_u [],double coord_v []) {
	if (mc_record) {
		FILE *fp;
		int j;
		double x,y,z;

		//save the state of the system to file//
		fp = fopen("history.csv","a");
		fprintf(fp,"%d,%ld,",*i_index_pt,n_particles);
		//printf("n_particles: %ld\n",n_particles);
	
		for (j=0;j<n_particles;j++) {
			cartesian(&x,&y,&z,coord_u[j],coord_v[j]);
			fprintf(fp,"%f,%f,%f,",x,y,z);
		}
		fprintf(fp,"\n");
		fclose(fp);
	}

	if (mc_record_energy) {
		FILE *fp2;
		int j;
		double energy = energy_total(coord_u,coord_v);
	
		fp2 = fopen("energy.csv","a");
		fprintf(fp2,"%d,%f\n",*i_index_pt,energy);
		fclose(fp2);
	}


}

/*records the number of nearest neighbours for each particles*/
void record_neighbours_PT (double coord_u[],double coord_v[]) {

	if (mc_record) {

		int i,j;

		int *neighbours = malloc(sizeof(int)*n_particles);
		for (i = 0; i < n_particles; i++) {
			neighbours[i] = 0;
		}
		double dist;
		FILE *fp;

		for (i = 0; i < n_particles-1; i++) {
			for (j = i+1;j< n_particles;j++) {
				dist = distance_NI_square (coord_u[i],coord_v[i],coord_u[j],coord_v[j]);
				if (dist < 2.0) {
					neighbours[i] ++;
					neighbours[j] ++;
				}
			}
		}

		fp = fopen("neighbours_history.csv","a");
		fprintf(fp,"%d,%ld,",*i_index_pt,n_particles);
		for (i = 0; i < n_particles; i++) {
			fprintf(fp,"%d,",neighbours[i]);
		}
		fprintf(fp,"\n");
		fclose(fp);
		free(neighbours);
	}
}


int exchange_down_temp (double *coord_u, double *coord_v) {
	

	/*get total energy by looping over all pairs*/
	int i;

	/*the total energy in units of epsilon (rather then the usual units of E/kT)*/
	double energy = 0;

	for (i = 0; i < n_particles; i++) {
		energy += energy_calc(coord_u,coord_v,coord_u[i],coord_v[i],i);
	}
	energy /= p_get_epsilon();

	//send energy, epsilon and current i
	double params[5];
	double paramsR[5];
	params[0] = energy;
	params[1] = p_get_epsilon();
	params[2] = (double) i_temperature;
	params[3] = delta_u;
	params[4] = delta_v;
	MPI_Status status;

	MPI_Ssend(params,5,MPI_DOUBLE,pt_rank-1,0,MPI_COMM_WORLD);
	MPI_Recv(paramsR,5,MPI_DOUBLE,pt_rank-1,1,MPI_COMM_WORLD,&status);

	p_set_epsilon ( paramsR[0] );
	i_temperature = (int) paramsR[1];
	delta_u = paramsR[3];
	delta_v = paramsR[4];

	if (paramsR[2] > 0.5 ) {
	  //  printf("D accept %f\n",paramsR[2]);
		return 1;
	} else {
	  //  printf("D reject %f\n",paramsR[2]);
		return 0;
	}
}

int exchange_up_temp(double *coord_u, double *coord_v) {

	/*get total energy by looping over all pairs*/
	int i;
	int success;
	/*the total energy in units of epsilon (rather then the usual units of E/kT)*/
	double energy = 0;

	for (i = 0; i < n_particles; i++) {
		energy += energy_calc(coord_u,coord_v,coord_u[i],coord_v[i],i);
	}
	energy /= p_get_epsilon();

	double paramsR[5];
	MPI_Status status;

	MPI_Recv(paramsR,5,MPI_DOUBLE,pt_rank+1,0,MPI_COMM_WORLD,&status);
	double params[5];

	if (randomF() < exp(-1*(p_get_epsilon()-paramsR[1])*(paramsR[0] - energy)) ) {
	  //	        printf("accept\n");
			success = 1;
		params[0] = p_get_epsilon();
		params[1] = (double) i_temperature;
		params[2] = (double) success;
		params[3] = delta_u;
		params[4] = delta_v;
		p_set_epsilon ( paramsR[1] );
		i_temperature = (int) paramsR[2];
		delta_u = paramsR[3];
		delta_v = paramsR[4];
	} else {
	  // 	        printf("reject\n");
			success = 0;
		params[0] = paramsR[1];
		params[1] = paramsR[2];
		params[2] = (double) success;
		params[3] = paramsR[3];
		params[4] = paramsR[4];
	}

	MPI_Ssend(params,5,MPI_DOUBLE,pt_rank+1,1,MPI_COMM_WORLD);
	return success;
}

int exchange_down_act(double *coord_u, double *coord_v) {

	/*get total energy by looping over all pairs*/
	int i;

	//send energy, epsilon and current i
	double params[5];
	double paramsR[5];
	params[0] = (double) n_particles;
	params[1] = mc_get_activity();
	params[2] = (double) i_activity;
	params[3] = delta_u;
	params[4] = delta_v;
	MPI_Status status;

	MPI_Ssend(params,5,MPI_DOUBLE,pt_rank-1,0,MPI_COMM_WORLD);
	MPI_Recv(paramsR,5,MPI_DOUBLE,pt_rank-1,1,MPI_COMM_WORLD,&status);

	mc_set_activity ( paramsR[0] );
	i_activity = (int) paramsR[1];
	delta_u = paramsR[3];
	delta_v = paramsR[4];

	if (paramsR[2] > 0.5 ) {
	  //  printf("D accept %f\n",paramsR[2]);
		return 1;
	} else {
	  //  printf("D reject %f\n",paramsR[2]);
		return 0;
	}
	


}

int exchange_up_act(double *coord_u, double *coord_v) {

	/*get total energy by looping over all pairs*/
	int i;
	int success;

	double paramsR[5];
	MPI_Status status;

	MPI_Recv(paramsR,5,MPI_DOUBLE,pt_rank+1,0,MPI_COMM_WORLD,&status);
	double params[5];

	if (randomF() < pow((mc_get_activity()/paramsR[1]),n_particles-(int)paramsR[0]) ) {
	  //	        printf("accept\n");
			success = 1;
		params[0] = mc_get_activity();
		params[1] = (double) i_activity;
		params[2] = (double) success;
		params[3] = delta_u;
		params[4] = delta_v;
		mc_set_activity ( paramsR[1] );
		i_activity = (int) paramsR[2];
		delta_u = paramsR[3];
		delta_v = paramsR[4];
	} else {
	  // 	        printf("reject\n");
			success = 0;
		params[0] = paramsR[1];
		params[1] = paramsR[2];
		params[2] = (double) success;
		params[3] = paramsR[3];
		params[4] = paramsR[4];
	}

	MPI_Ssend(params,5,MPI_DOUBLE,pt_rank+1,1,MPI_COMM_WORLD);
	return success;

}







