#ifndef MONTE_CARLO_PT
#define MONTE_CARLO_PT

/* a module to contain functions for hte various monte carlo 
 * steps, including parellel tempering. They take the system, make a trail move and 
 * correctly accept or reject it. INCOMPATIBLE with monte_carlo module!! this is an
 * alternative.
 * this module is general. the supermodule monte_carlo_sphere exists for the special
 * spherical case
 *
 * the prefix is mcpt_
 */

#include"monte_carlo_h.h"
#include"mpi_h.h"
#include"pt_init_h.h"
#include"geometry_h.h"
#include"energy_h.h"

#include"std_h.h"
#include"mc_math_h.h"
#include"keywords.c"
#include"twister.c"
#include"n_particles.h"

/*the current temperature index*/
int i_temperature;

/*the current activity index*/
int i_activity;

/*a pointer to the correct index*/
int *i_index_pt;

/*getters and setters for swapping variable*/
void (*set_swap) (double u);
double (*get_swap)();

/*read file for monte carlo*/
void mcpt_read (FILE *infile);

//saves the current state to disc, overrighting previous
void save_PT(double coord_u [],double coord_v []);

//records the current state in cartesians, does not overwrite, but appends
void record_PT(double coord_u [],double coord_v []);

void record_neighbours_PT (double coord_u[],double coord_v[]);

/*Function exhange down does a parellel exchange with the system at a lower rank. It
 * sends it's swapping vaiable, index and total energy and waits to recieve it's new ones 
 */
int (*exchange_down)(double *coord_u, double * coord_v);

/*Function exhange up does a parellel exchange with the system at a higher rank. It
 * recieves a swapping variable and total energy, then accepts or rejects, then send back
 * the new value
 */
int (*exchange_up) (double *coord_u, double *coord_v);

/*
 * FUNCTION particle_move: this function takes a particle and performs a
 * single particle move monte carlo step. used for most ensembles (esp canonical)
 *
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

/*int particle_move (double coord_u[], double coord_v[]);*/

/*
 * FUNCTION particle_add: performs a grand canonical attempted particle addition
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

/*int particle_add (double **coord_u, double **coord_v);*/

/*
 * FUNCTION particle_sub: performs a grand canonical attempted particle removal
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

/*int particle_sub (double *coord_u[], double *coord_v[]);*/

#include"monte_carlo_PT.c"
#endif
