#ifndef MONTE_CARLO
#define MONTE_CARLO

/* a module to contain functions for hte various monte carlo 
 * steps. They take the system, make a trail ove and correctly accept or reject it
 * this module is general. the supermodule monte_carlo_sphere exists for the special
 * spherical case
 *
 * the prefix is mc_
 */

#include"move_h.h"
#include"energy_h.h"
#include"geometry_h.h"

#include"std_h.h"
#include"mc_math_h.h"
#include"keywords.c"
#include"twister.c"
#include"n_particles.h"

/*global*/
/*The initial number of particles, a the start of the simulation*/
long n_particles_initial;

/*read file for monte carlo*/
void mc_read (FILE *infile);

//saves the current state to disc, overrighting previous
void save(double coord_u [],double coord_v []);

//records the current state in cartesians, does not overwrite, but appends
void record(double coord_u [],double coord_v []);

//records the total energy of the system
void record_energy(double coord_u [],double coord_v []);

/*records the number of nearest neighbours for each particles*/
void record_neighbours (double coord_u[],double coord_v[]);


/*setter and getter*/
void mc_set_activity(double u);
double mc_get_activity();

/*
 * FUNCTION particle_move: this function takes a particle and performs a
 * single particle move monte carlo step. used for most ensembles (esp canonical)
 *
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int particle_move (double coord_u[], double coord_v[]);

/*
 * FUNCTION particle_add: performs a grand canonical attempted particle addition
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int particle_add (double **coord_u, double **coord_v);

/*
 * FUNCTION particle_sub: performs a grand canonical attempted particle removal
 * 
 * OUTPUT int: 1 if the move was accepted, 0 if rejected
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int particle_sub (double *coord_u[], double *coord_v[]);

#include"monte_carlo.c"
#endif
