/*-----global-----*/
double mp_epsilon;
double mp_rho;
double mp_cuttoff = 1000;
double mp_at_cut = 0;
double d_mp_at_cut = 0;

/*read function for morse*/
void mp_read (FILE *infile) {

	char command [50];
	char error[20];
	
	int success_e = 0 ;
	int success_r =0;

	/*read in values*/

	if (infile == NULL) die ("Could not open options file!");
	rewind(infile);
	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"EPSILON")==0) {
			if (!get_double(&mp_epsilon)) die ("Could not read epsilon");
			success_e = 1;
		} else if (strcmp(command,"RHO")==0) {
			if (!get_double(&mp_rho)) die ("could not read rho");
			success_r = 1;
		} else if (strcmp(command,"CUTTOFF")==0) {
			if (!get_double(&mp_cuttoff)) die ("could not read cuttoff");
		}
	}


	if (success_e == 0) die ("epsilon not defined in options file");
	if (success_r == 0) die ("rho not defined in options file");	

	if (mp_epsilon < 0) {
		die("the well depth should be a positive number");
	}

	if (mp_rho < 0) {
		die("the potential range should be a positive integer");
	}

	double energy = exp( mp_rho*(1 - mp_cuttoff ) );
	mp_at_cut = mp_epsilon*energy*(energy-2.0);
	d_mp_at_cut = -mp_epsilon*mp_rho*2*energy*(energy-1);

}


/*
 FUNCTION: double morse: calculates the pairwise morse potential
 * OUTPUT: morse potential in units of kt
 * INPUT: the separation of the two particles
 */

double morse (double distance) {


	if (distance > mp_cuttoff) {
		return 0;
	} else {
		double energy = exp( mp_rho*(1 - distance ) );
		return mp_epsilon*energy*(energy-2.0) - mp_at_cut + (distance-mp_cuttoff)*d_mp_at_cut;
	}
}




