
/*this module contains functions for moving particles around,
 * all the function in this module are general to any geometry, but
 * this module is often used as part of modules which include geometry specific
 * moves.
 */

/*reader for move*/
void m_read(FILE *infile) {
	g_read(infile);
	delta_u = (max_u-min_u)/300;
	delta_v = (max_v-min_v)/300;
}

/* FUNCTION: move moves a single particle with current general coordinates
 * u,v to a new positions at random. The probability is uniformly distributed
 * from u-m_delta_u to u+m_delta_u and v-m_delta_v to v+m_delta_v
 *
 * Output: 0 if the move must be rejected (ie, it goes beyond a reflective boundry)
 *	 : 1 if the move is allowable
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 * Input: double m_delta_u: the maximum change in the u coordinate (see above)
 * Input: double dleta_v: the maximum change in the v coordinate (see above)
 */

int move (double *u, double *v) {

	double o_u = *u;
	double o_v = *v;
	double r = randomF();
	*u +=  (2*r-1)*delta_u;
	r = randomF();
	*v +=  (2*r-1)*delta_v;
	

	if (*u > max_u) {
		if (bound_u == 0) {
		*u -= max_u-min_u;
		} else if (bound_u == 1){
			*u = o_u;
			*v = o_v;
			return 0;
		}
	} 

	if (*u < min_u) {
		if (bound_u == 0) {
		*u += max_u-min_u;
		} else if (bound_u == 1) {
			*u = o_u;
			*v = o_v;
			return 0;
		}
	}

	if (*v > max_v) {
		if (bound_v == 0) {
		*v -= max_v-min_v;
		} else if (bound_v == 1){
			*u = o_u;
			*v = o_v;
			return 0;;
		}
	} 

	if (*v < min_v) {
		if (bound_v == 0) {
		*v += max_v-min_v;
		} else if (bound_v == 1){
			*u = o_u;
			*v = o_v;
			return 0;
		}
	}

	return 1;
}
