/*
 * a module containing functions which perform special moves on a flat plane
 * a supermodule of move.c. on the plane,u and v are the x and y coordinates 
 * respectively
 *
 * The prefix is mf_ 
 */

void mf_read(FILE *infile) {
	m_read(infile);
	delta_cluster = (max_u-min_u)/300;
	delta_spin = 2*M_PI/300;

}

/* FUNCTION: move_cluster takes a whole cluster and moves it as a whole. The new
 * position of the centre of the mass of the cluster moves
 * by distance less than delta_cluster.
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

void mf_move_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]) { 

	
	double o_u,o_v;
	double dy = ( 2*randomF()-1 )* delta_cluster;
	double dx = ( 2*randomF()-1 )* delta_cluster;

	int n;
	for (n = head; n != -1; n=ll_clusters[n]) {
		
		o_u = coord_u[n];
		o_v = coord_v[n];
		coord_u[n] += dx;
		coord_v[n] += dy;

		if (coord_u[n] > max_u) {
			if (bound_u == 0) {
			coord_u[n] -= max_u-min_u;
			} else if (bound_u == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		} 

		if (coord_u[n] < min_u) {
			if (bound_u == 0) {
			coord_u[n] += max_u-min_u;
			} else if (bound_u == 1) {
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		}

		if (coord_v[n] > max_v) {
			if (bound_v == 0) {
			coord_v[n] -= max_v-min_v;
			} else if (bound_v == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		} 

		if (coord_v[n] < min_v) {
			if (bound_v == 0) {
			coord_v[n] += max_v-min_v;
			} else if (bound_v == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		}

	} 

}

/* FUNCTION: spin_cluster takes a whole cluster and spins it as a whole. The new
 * position of the centre of the mass of the cluster does not change
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */
void mf_spin_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]) { 

	// find COM
	int n;
	int n_in_cluster = 0;
	double u_com,v_com,o_u,o_v;
	double angle = (randomF()-0.5)*delta_spin;
	u_com = 0;
	v_com = 0;
	for (n=head; n != -1; n= ll_clusters[n]) {
		u_com += coord_u[n];
		v_com += coord_v[n];
		n_in_cluster++;
		
	}
	u_com = u_com/ n_in_cluster;
	v_com = v_com / n_in_cluster;

	//tranform cluster to the origin, rotate, restore
	for (n=head; n != -1; n = ll_clusters[n]) {

		o_u = coord_u[n];
		o_v = coord_v[n];

		coord_u[n] -= u_com;
		coord_v[n] -= v_com;
		coord_u[n] = coord_u[n]*cos(angle)-coord_v[n]*sin(angle);
		coord_v[n] = coord_v[n]*cos(angle)+coord_u[n]*sin(angle);
		coord_u[n] += u_com;
		coord_v[n] += v_com;
		if (coord_u[n] > max_u) {
			if (bound_u == 0) {
			coord_u[n] -= max_u-min_u;
			} else if (bound_u == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		} 
		if (coord_u[n] < min_u) {
			if (bound_u == 0) {
			coord_u[n] += max_u-min_u;
			} else if (bound_u == 1) {
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		}
		if (coord_v[n] > max_v) {
			if (bound_v == 0) {
			coord_v[n] -= max_v-min_v;
			} else if (bound_v == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		} 
		if (coord_v[n] < min_v) {
			if (bound_v == 0) {
			coord_v[n] += max_v-min_v;
			} else if (bound_v == 1){
				coord_u[n] = o_u;
				coord_v[n] = o_v;
			}
		}	

	}




}



