#ifndef MOVE_H
#define MOVE_H

/*this module contains functions for moving particles around,
 * all the function in this module are general to any geometry, but
 * this module is often used as part of modules which include geometry specific
 * moves.
 * the prefix for this module us m_
 */
#include"twister.c"
#include"std_h.h"
#include"mc_math_h.h"
#include"geometry_h.h"

/*globals*/
/*the amount in u that a particle can move per step*/
double delta_u;
/*the amount in v that a particle can move per step*/
double delta_v;

/*reader for move*/
void m_read(FILE *infile);

/*
 * FUNCTION get random position. Gets a random particle location between
 * that satifies the allowed u,v ranges
 *
 * Param double *u,*v: locations to store the new particle coords
 */

void random_position (double *u, double *v);

/* FUNCTION: move moves a single particle with current general coordinates
 * u,v to a new positions at random. The probability is uniformly distributed
 * from u-delta_u to u+delta_u and v-delta_v to v+delta_v
 *
 * Output: 0 if the move must be rejected (ie, it goes beyond a reflective boundry)
 *	 : 1 if the move is allowable
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

int move (double *u, double *v);


#include"move.c"
#endif
