/*a module containing functions which perform special moves on a sphere
 * a supermodule of move.c. On a sphere, the u coordinate is the "phi" 
 * sphereical coodinate and the v coordinate is cos(theta)
 *
 * the prefix for this module is _ms*/

/*read file*/
void ms_read(FILE *infile) {
	m_read(infile);
	delta_cluster = 2*M_PI/300;
	delta_spin = 2*M_PI/300;
}

/* FUNCTION: move_cluster takes a whole cluster and moves it as a whole. The new
 * position of the centre of the mass of the cluster moves arounnd the sphere
 * by an angle less than delta_cluster.
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

void ms_move_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]) { 

///find COM
///rotate coordinates
///change phi
///rotate back

double x,y,z,x_com = 0,y_com = 0,z_com = 0,u_com = 0,v_com =0;
double x_prime,y_prime,z_prime,u_prime,v_prime,x_prime_r,y_prime_r,z_prime_r;
int n_in_cluster=0;
int n;

//Centre of mass
n = head;
while (n != -1) {

	cartesian (&x,&y,&z,coord_u[n],coord_v[n]);
	x_com += x;
	y_com += y;
	z_com += z;

	n_in_cluster++;
	n = ll_clusters[n];
}

x_com = x_com/ n_in_cluster;
y_com = y_com / n_in_cluster;
z_com = z_com / n_in_cluster;



uv_coords(&u_com,&v_com,x_com,y_com,z_com);


double move_angle = randomF()*2*M_PI;
double move_length = (2*randomF()-1)*delta_cluster;

//matrix elements//
double xx = sin(acos(v_com))*cos(u_com);
//printf("xx:%f\n",xx);
double xy = sin(acos(v_com))*sin(u_com);
//printf("xy:%f\n",xy);
double yx = -sin(u_com)*cos(move_angle) - v_com*cos(u_com)*sin(move_angle);
//printf("yx:%f\n",yx);
double yy = cos(move_angle)*cos(u_com) - sin(move_angle)*v_com*sin(u_com);
//printf("yy:%f\n",yy);
double yz = sin(move_angle)*sin(acos(v_com));
//printf("yz:%f\n",yz);
double zx = sin(u_com)*sin(move_angle) - v_com*cos(u_com)*cos(move_angle);
//printf("zx:%f\n",zx);
double zy = -sin(move_angle)*cos(u_com) - cos(move_angle)*v_com*sin(u_com);
//printf("zy:%f\n",zy);
double zz = cos(move_angle)*sin(acos(v_com));
//printf("zz:%f\n",zz);


/////move all the particles////
n = head;
while (n != -1) {
	
	cartesian (&x,&y,&z,coord_u[n],coord_v[n]);
	//printf("1:%f,%f,%f\n",x,y,z);
	x_prime = xx*x + xy*y + v_com*z;
	y_prime = yx*x + yy*y + yz*z;
	z_prime = zx*x + zy*y + zz*z;
	//printf("2:%f,%f,%f\n",x_prime,y_prime,z_prime);	

	x_prime_r = (cos(move_length)*x_prime - sin(move_length)*y_prime);
	y_prime_r = (sin(move_length)*x_prime + cos(move_length)*y_prime);
	z_prime_r = z_prime;
	//printf("%f,%f\n",cos(move_length),sin(move_length));
	//rotate back
	//printf("3:%f,%f,%f\n",x_prime_r,y_prime_r,z_prime_r);
	x = xx*x_prime_r + yx*y_prime_r + zx*z_prime_r;
	y = xy*x_prime_r + yy*y_prime_r + zy*z_prime_r;
	z = v_com*x_prime_r + yz*y_prime_r + zz*z_prime_r;
	//printf("4:%f,%f,%f\n",x,y,z);
	uv_coords(coord_u+n,coord_v+n,x,y,z);
	//printf("%f,%f\n",coord_u[n],coord_v[n]);
	n = ll_clusters[n];
}


}


/* FUNCTION: spin_cluster takes a whole cluster and spins it as a whole. The new
 * position of the centre of the mass of the cluster does not change
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */
void ms_spin_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]) { 

///find COM
///rotate coordinates
///change phi
///rotate back

double x,y,z,x_com = 0,y_com = 0,z_com = 0,u_com = 0,v_com =0;
double x_prime,y_prime,z_prime,u_prime,v_prime,x_prime_r,y_prime_r,z_prime_r;
int n_in_cluster=0;
int n;


//Centre of mass
n = head;
while (n != -1) {

	cartesian (&x,&y,&z,coord_u[n],coord_v[n]);
	x_com += x;
	y_com += y;
	z_com += z;

	n_in_cluster++;
	n = ll_clusters[n];
}

x_com = x_com/ n_in_cluster;
y_com = y_com / n_in_cluster;
z_com = z_com / n_in_cluster;

uv_coords(&u_com,&v_com,x_com,y_com,z_com);

double move_length = (randomF()-0.5)*delta_spin;

//matrix elements//
//matrix elements//
//matrix elements//
double xx = sin(acos(v_com))*cos(u_com);
//printf("xx:%f\n",xx);
double xy = sin(acos(v_com))*sin(u_com);
//printf("xy:%f\n",xy);
double yx = -sin(u_com);
//printf("yx:%f\n",yx);
double yy = cos(u_com);
//printf("yy:%f\n",yy);
//printf("yz:%f\n",yz);
double zx = - v_com*cos(u_com);
//printf("zx:%f\n",zx);
double zy = -v_com*sin(u_com);
//printf("zy:%f\n",zy);
double zz = sin(acos(v_com));
//printf("zz:%f\n",zz);

/////move all the particles////
n = head;
while (n != -1) {
	
	cartesian (&x,&y,&z,coord_u[n],coord_v[n]);
	//printf("1:%f,%f,%f\n",x,y,z);
	x_prime = xx*x + xy*y + v_com*z;
	y_prime = yx*x + yy*y ;
	z_prime = zx*x + zy*y + zz*z;
	//printf("2:%f,%f,%f\n",x_prime,y_prime,z_prime);	

	x_prime_r = x_prime;
	y_prime_r = cos(move_length)*y_prime - sin(move_length)*z_prime;
	z_prime_r = sin(move_length)*y_prime + cos(move_length)*z_prime;
	//printf("%f,%f\n",cos(move_length),sin(move_length));
	//rotate back
	//printf("3:%f,%f,%f\n",x_prime_r,y_prime_r,z_prime_r);
	x = xx*x_prime_r + yx*y_prime_r + zx*z_prime_r;
	y = xy*x_prime_r + yy*y_prime_r + zy*z_prime_r;
	z = v_com*x_prime_r + zz*z_prime_r;
	//printf("4:%f,%f,%f\n",x,y,z);
	uv_coords(coord_u+n,coord_v+n,x,y,z);
	//printf("%f,%f\n",coord_u[n],coord_v[n]);
	n = ll_clusters[n];
}

}

