#ifndef MOVE_SPHERE_H
#define MOVE_SPHERE_H

/*a module containing functions which perform special moves on a sphere
 * a supermodule of move.c. On a sphere, the u coordinate is the "phi" 
 * sphereical coodinate and the v coordinate is cos(theta)
 *
 * the prefix for this module is _ms*/
#include"move_h.h"
#include"geometry_h.h"

#include"std_h.h"
#include"mc_math_h.h"
#include"twister.c"

extern double delta_cluster;
extern double delta_spin;

/*read file*/
void ms_read(FILE *infile);

/*read function is the same as the in in geometery.h. Remember to call
 * void g_read(infile);
 **/

/* FUNCTION: move_cluster takes a whole cluster and moves it as a whole. The new
 * position of the centre of the mass of the cluster moves arounnd the sphere
 * by an angle less than delta_cluster.
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */

void ms_move_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]);

/* FUNCTION: spin_cluster takes a whole cluster and spins it as a whole. The new
 * position of the centre of the mass of the cluster does not change
 *
 * Input int head: the index of the particle at the head of the cluster to be moved
 * Input int ll_cluster[]: a linked list. the value at the index of a given particle
 * is the index of the next particle in the list. Defines the clusters.
 * Param: double u* a pointer to the location which stores the particles u coord
 * Param: double v* a pointer to the location which stores the particles v coord
 */
void ms_spin_cluster (int head, int ll_clusters[],double coord_u[],double coord_v[]);


#include"move_sphere.c"
#endif
