#ifndef POTENTIAL_H
#define POTENTIAL_H

/* A module for dealing with the potentialfelt by particles in the system
 *
 * The prefix for this module is _p
 */
#include"lennard_jones_h.h"
#include"morse_h.h"
#include"geometry_h.h"

#include"std_h.h"
#include"keywords.c"

/*setters*/
void p_set_epsilon(double u);

/*getter*/
double p_get_epsilon();

/* FUNCTION POINTER potential: a pointer to a function that gets the pairwise
 * potential between two particles
 *
 *OUTPUT: the potential in units of kT
 *INPUT: double distance: often the cartesian distance between the two particles
 * potential dependant, see bellow
 */

double (*potential) (double distance);

/*
 * FUNCTION POINTER distance_pot: a function that gets the argument required for
 * "potential". Geometry and potential dependent dependent
 *
 * OUTPUT:the arguments for "potential"
 * INPUT u1,u2,v1,v2 the generalised coordinates for the two particles
 */

double (*distance_pot) (double u1, double v1, double u2, double v2);

/*read function*/
void p_read(FILE *infile);


#include"potential.c"
#endif
