/*a module containing the basic geometry functions required to work in
 * gerneralised sphereical coordinates.
 * u: the sphereical polar angle phi
 * v: the COSINE of the polar angle theta
 *
 * the prefix for this module is _s
 */

/*-----globals-------*/
double s_radius;


/*read function for sphere*/
void s_read (FILE *infile) {

	char command [50];
	char error[20];
	
	int success = 0 ;
	rewind(infile);
	/*read in values*/
	if (infile == NULL) die ("Could not open options file!");

	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"RADIUS")==0) {
			if (!get_double(&s_radius)) die ("Could not read radius");
			success = 1;
		}
	}

	if (success == 0) die ("radius not defined in options file");
	if (s_radius < 0.99999) {
		printf("%f\n",s_radius);
		die("the radius of the surface should be larger than 1: the bond length");
	}
	
}

/*FUNCTION: cartesian is a function which converts from the current
 * to cartesians.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void s_cartesian (double *x, double *y, double *z,double coord_u, double coord_v) {

	*x = s_radius*sin(acos(coord_v))*cos(coord_u);
	*y = s_radius*sin(acos(coord_v))*sin(coord_u);
	*z = s_radius*coord_v ;
}

/*FUNCTION: cartesian_bx is a function which converts from the current
 * to cartesians, but shifts the origin so that no coordinates are negative.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void s_cartesian_bx (double *x, double *y, double *z,double coord_u, double coord_v) {
	
	*x = s_radius*sin(acos(coord_v))*cos(coord_u) + s_radius;
	*y = s_radius*sin(acos(coord_v))*sin(coord_u) + s_radius;
	*z = s_radius*coord_v + s_radius;
}

/* FUNCTION: distance_NI returns the cartesian distance between two
 * particles. Under periodic bounderies, the nearest image convention is used.
 *
 * Output: double, The particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double s_distance_NI (double u1,double v1, double u2, double v2) {
	return s_radius*sqrt(2* (1-v1*v2-sin(acos(v1))*sin(acos(v2))*cos(u1-u2)));
}

/* FUNCTION: distance_NI_2 returns the square of the cartesian distance
 * between two particles. Faster than sqrt(distance_NI()).
 *
 * Output: double, The square of the particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double s_distance_NI_square(double u1,double v1, double u2, double v2) {
	return s_radius*s_radius*(2* (1-v1*v2-sin(acos(v1))*sin(acos(v2))*cos(u1-u2)));
}

/* FUNCTION: uv_coords converts cartesian coordinates to generalised coordinates
 *
 * Input: double x,y,z the cartsian coordinates of the particle
 * Output: double *u,*v, pointers to a location to contain the generalised coordinates u,v
 */
void s_uv_coords(double *u, double *v, double x, double y, double z) {

	*u = atan2(y,x);
	if(*u<0) *u+=2*M_PI;
	*v = z/s_radius;
}





