#ifndef SPHERE_H
#define SPHERE_H

/*a module containing the basic geometry functions required to work in
 * gerneralised sphereical coordinates.
 * u: the sphereical polar angle phi
 * v: the COSINE of the polar angle theta
 *
 * the prefix for this module is _s
 */

#include"std_h.h"
#include"mc_math_h.h"
#include"keywords.c"

/*read function for sphere*/
void s_read (FILE *infile);

/*FUNCTION: cartesian is a function which converts from the current
 * to cartesians.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void s_cartesian(double *x, double *y, double *z,double coord_u, double coord_v);

/*FUNCTION: cartesian_bx is a function which converts from the current
 * to cartesians, but shifts the origin so that no coordinates are negative.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void s_cartesian_bx(double *x, double *y, double *z,double coord_u, double coord_v);

/* FUNCTION: distance_NI returns the cartesian distance between two
 * particles. Under periodic bounderies, the nearest image convention is used.
 *
 * Output: double, The particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double s_distance_NI (double u1,double v1, double u2, double v2);

/* FUNCTION: distance_NI_2 returns the square of the cartesian distance
 * between two particles. Faster than sqrt(distance_NI()).
 *
 * Output: double, The square of the particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double s_distance_NI_square(double u1,double v1, double u2, double v2);

/* FUNCTION: uv_coords converts cartesian coordinates to generalised coordinates
 *
 * Input: double x,y,z the cartsian coordinates of the particle
 * Output: double *u,*v, pointers to a location to contain the generalised coordinates u,v
 */
void s_uv_coords(double *u, double *v, double x, double y, double z);


#include"sphere.c"
#endif
